import os
from os import listdir
from os.path import isfile, join

from Homevee.Utils import Constants


class DBMigration():
    def __init__(self):
        self.scripts_dir = os.path.join(Constants.HOMEVEE_DIR, "DBMigration", "scripts")

    def get_scripts(self):
        """
        Get all migration scripts in alphabetical order
        :return: list of filenames as strings
        """
        try:
            files = [f for f in listdir(self.scripts_dir) if isfile(join(self.scripts_dir, f))]
            return sorted(files)
        except:
            return []

    def get_filecontent_version_map(self):
        """
        Get a map of versions and script content
        :return: a map of the format version(int) => content(string)
        """
        files = self.get_scripts()
        version_map = {}
        for file in files:
            script_version = int(file.split("_")[0])
            script_content = self.read_script_content(file)
            version_map[script_version] = script_content
        return version_map

    def read_script_content(self, file_name: str):
        """
        Get the script content
        :param file_name: the filename as a string
        :return: the script content
        """
        with open(os.path.join(self.scripts_dir, file_name), 'r') as file:
            return file.read().replace('\n', '')

if __name__ == "__main__":
    migration = DBMigration()
    print(migration.get_filecontent_version_map())


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Functions.sensor_data import get_einheit
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database

'''Gibt den Sensorwerteverlauf für den gewünschten Zeitraum zurück'''
def get_graph_data(user, room, type, id, von, bis, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    if von == bis:
        datum = datetime.datetime.strptime(von, "%d.%m.%Y").strftime("%Y-%m-%d")
        return get_day_data(type, id, datum, db)
    else:
        return get_day_min_max(type, id, von, bis, db)

'''Gibt den Tageswerteverlauf des Sensors zurück'''
def get_day_data(type, id, datum, db: Database = None):
    if db is None:
        db = Database()
    results = db.select_all("""SELECT * FROM 'SENSOR_DATA' WHERE DEVICE_TYPE == :type AND DEVICE_ID == :id 
            AND DATETIME >= :start AND DATETIME < :ende AND VALUE != \"N/A\" ORDER BY DATETIME ASC""",
                {'type':type, 'id':id, 'start':datum+" 00:00", 'ende':datum+" 23:59"})
    values = []

    for data in results:
        item = {'value':float(data['VALUE']), 'id':id, 'time':data['DATETIME'].replace(datum+" ","")}
        values.append(item)

    return {'values': values, 'einheit': get_einheit(type, id)}

'''Gibt die Höchst- & Tiefstwerte des angegebenen Datumsbereichs zurück'''
def get_day_min_max(type, id, von, bis, db: Database = None):
    if db is None:
        db = Database()
    start = datetime.datetime.strptime(von, "%d.%m.%Y").strftime("%Y-%m-%d")
    ende = datetime.datetime.strptime(bis, "%d.%m.%Y").strftime("%Y-%m-%d")

    values = []

    results = db.select_all("""SELECT MIN(VALUE) as MIN, MAX(VALUE) as MAX, strftime(\"%d.%m.%Y\", DATETIME) as FORMATTED_DATE
         FROM SENSOR_DATA WHERE DEVICE_TYPE == :type AND DEVICE_ID == :id AND DATETIME >= :start 
         AND DATETIME <= :ende AND VALUE != \"N/A\" GROUP BY FORMATTED_DATE ORDER BY DATETIME ASC""",
        {'type': type, 'id': id, 'start': start+' 00:00', 'ende': ende+' 23:59'})

    #print {'type': type, 'id': id, 'start': start, 'ende': ende}

    for data in results:
        item = {'date':data['FORMATTED_DATE'], 'min':data['MIN'], 'max':data['MAX']}

        values.append(item)

    return {'values':values, 'einheit': get_einheit(type, id)}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import sqlite3
import traceback

from Homevee.DBMigration import DBMigration
from Homevee.Helper import Logger
from Homevee.Utils import Constants


# use encrypted databases
# http://charlesleifer.com/blog/encrypted-sqlite-databases-with-python-and-sqlcipher/

class Database():
    def __init__(self):
        self.db_con = self.get_database_con()

    @staticmethod
    def dict_factory(cursor, row):
        d = {}
        for idx, col in enumerate(cursor.description):
            d[col[0]] = row[idx]
        return d

    def get_database_con(self):
        db_path = os.path.join(Constants.DATA_DIR, "data.db")

        # init db if not yet existing
        if not os.path.isfile(db_path):
            open(db_path, "w+")
            self.upgrade()
        con = sqlite3.connect(db_path)
        con.text_factory = str
        con.row_factory = Database.dict_factory

        return con

    def get_server_data(self, key):
        item = self.select_one("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                    {'key': key})

        try:
            return item['VALUE']
        except Exception as e:
            return None

    def set_server_data(self, key, value):
        self.insert("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES(:value, :key)",
                    {'value': value, 'key': key})

        self.update("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key",
                    {'value': value, 'key': key})

    def do_query(self, query, params=None):
        Logger.log(("DATABASE_QUERY", query, params))

        if params is None:
            params = {}

        with self.db_con:
            cur = self.db_con.cursor()
            cur.execute(query, params)
            return cur

    def select_one(self, query, params=None):
        try:
            cur = self.do_query(query, params)
            output = cur.fetchone()
            cur.close()
            return output
        except:
            return None

    def select_all(self, query, params=None):
        try:
            cur = self.do_query(query, params)
            output = cur.fetchall()
            cur.close()
            return output
        except:
            return []

    def insert(self, query, params=None):
        try:
            cur = self.do_query(query, params)
            last_id = cur.lastrowid
            cur.close()
            return last_id
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False

    def update(self, query, params=None):
        try:
            cur = self.do_query(query, params)
            cur.close()
            return True
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False

    def delete(self, query, params=None):
        try:
            cur = self.do_query(query, params)
            cur.close()
            return True
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False

    @staticmethod
    def upgrade():
        """
        Upgrades the database to the current scheme
        :return:
        """
        Logger.log("Upgrading Database...")
        db = Database()
        with db.db_con:
            cur = db.db_con.cursor()
            migration = DBMigration()
            try:
                db_version = int(db.get_server_data("DB_VERSION"))
            except:
                db_version = 0
            db_script_map = migration.get_filecontent_version_map()
            db_script_versions = db_script_map.keys()
            last_version = 0
            for version in db_script_versions:
                if (version <= db_version):
                    continue
                try:
                    Logger.log("Executing DB-Upgrade-Script V" + str(version) + "...")
                    cur.executescript(db_script_map[version])
                except:
                    db.set_server_data("DB_VERSION", version)
            cur.close()


import traceback

from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database


class CalendarEntry(Item):
    def __init__(self, name, date, start, end, note, address, id=None):
        super(CalendarEntry, self).__init__()
        self.name = name
        self.date = date
        self.start = start
        self.end = end
        self.note = note
        self.address = address
        self.id = id

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        return db.delete("DELETE FROM CALENDAR WHERE ID == :id", {'id': self.id})

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                new_id = db.insert("""INSERT INTO CALENDAR (NAME, START, END, NOTE, DATE, ADDRESS) VALUES 
                (:name, :start, :end, :note, :date, :address)""",
                                {'name': self.name, 'date': self.date, 'start': self.start, 'end': self.end,
                                 'note': self.note, 'address': self.address})
                self.id = new_id
            #update
            else:
                db.update("""UPDATE CALENDAR SET NAME = :name, START = :start, END = :end, NOTE = :note, 
                DATE = :date, ADDRESS = :address WHERE ID = :id""",
                                {'name': self.name, 'date': self.date, 'start': self.start, 'end': self.end,
                                 'note': self.note, 'address': self.address, 'id': self.id})
                # TODO add generated id to object
        except:
            if (Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save calendar-entry to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'name': self.name,
            'date': self.date,
            'start': self.start,
            'end': self.end,
            'address': self.address,
            'note': self.note,
        }

        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = CalendarEntry(result['NAME'], result['DATE'], result['START'], result['END'],
                                 result['NOTE'], result['ADDRESS'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all_by_date(date, db: Database = None):
        if db is None:
            db = Database()
        return CalendarEntry.load_all_from_db('SELECT * FROM CALENDAR WHERE DATE = :date ORDER BY START ASC',
                                              {'date': date})

    @staticmethod
    def load_all(db: Database):
        if db is None:
            db = Database()
        return CalendarEntry.load_all_from_db('SELECT * FROM CALENDAR ORDER BY START ASC', {}, db)

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return CalendarEntry.load_all_from_db('SELECT * FROM CALENDAR WHERE ID IN (%s)'
                                       % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            name = dict['name']
            date = dict['date']
            start = dict['start']
            end = dict['end']
            address = dict['address']
            note = dict['note']

            item = CalendarEntry(name, date, start, end, note, address, id)

            return item
        except:
            raise InvalidParametersException("CalendarEntry.create_from_dict(): invalid dict")

    @staticmethod
    def get_calendar_item_dates(year, db: Database = None):
        if db is None:
            db = Database()
        calendar_item_dates = []

        results = db.select_all("SELECT DISTINCT DATE FROM CALENDAR WHERE strftime('%Y', DATE) = :year",
                                      {'year': year})

        for calendar_entry in results:
            calendar_item_dates.append(calendar_entry['DATE'])

        return {'dates': calendar_item_dates}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_place(id, db: Database = None):
    if db is None:
        db = Database()
    if(id is None or id == "" or id is -1):
        return None

    place = db.select_one("SELECT * FROM PLACES WHERE ID = :id",
                {'id': id})

    return place

def get_my_places(username, db: Database = None):
    if db is None:
        db = Database()
    places = []

    results = db.select_all("SELECT * FROM PLACES", {})

    for place in results:
        places.append({'id': place['ID'], 'name': place['NAME'], 'address': place['ADDRESS'], 'latitude': place['LATITUDE'], 'longitude': place['LONGITUDE']})

    return {'places': places}

def add_edit_place(username, id, name, address, latitude, longitude, db: Database = None):
    if db is None:
        db = Database()
    add_new = (id == None or id == "" or id == "-1")

    if (add_new):
        db.insert("INSERT INTO PLACES (NAME, ADDRESS, LATITUDE, LONGITUDE) VALUES (:name, :address, :latitude, :longitude)",
                    {'name': name, 'address': address, 'latitude': latitude, 'longitude': longitude})

        return Status(type=STATUS_OK).get_dict()

    else:
        db.update("UPDATE SCENES SET NAME = :name, ADDRESS = :address, LATITUDE = :latitude, LONGITUDE = :longitude WHERE ID = :id",
                    {'name': name, 'address': address, 'latitude': latitude, 'longitude': longitude, 'id': id})

        return Status(type=STATUS_OK).get_dict()

def delete_place(username, id, db: Database = None):
    if db is None:
        db = Database()
    db.delete("DELETE FROM PLACES WHERE ID = :id", {'id': id})
    return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item import Item
from Homevee.Item.CalendarEntry import CalendarEntry
from Homevee.Utils.Database_NEW import Database


def get_calendar_item_dates(user, year, db: Database = None):
    if db is None:
        db = Database()
    return CalendarEntry.get_calendar_item_dates(year)

def get_calendar_day_items(user, date, db: Database = None):
    if db is None:
        db = Database()
    calendar_items = CalendarEntry.load_all_by_date(date)
    return {'calendar_entries': CalendarEntry.list_to_dict(calendar_items)}

def delete_entry(user, id, db: Database = None):
    if db is None:
        db = Database()
    calendar_entry = Item.load_from_db(CalendarEntry, id)
    return calendar_entry.api_delete(db)

def add_edit_entry(user, entry_id, name, date, start, end, note, address, db: Database = None):
    if db is None:
        db = Database()
    try:
        calendar_entry = Item.load_from_db(CalendarEntry, entry_id)
        calendar_entry.name = name
        calendar_entry.date = date
        calendar_entry.start = start
        calendar_entry.end = end
        calendar_entry.note = note
        calendar_entry.address = address
    except:
        calendar_entry = CalendarEntry(name, date, start, end, note, address)

    return calendar_entry.api_save(db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Cronjob import FixedTimeCronjob
from Homevee.Item.Device.Sensor.MQTTSensor import MQTTSensor
from Homevee.Item.Device.Sensor.ZWaveSensor import ZWaveSensor
from Homevee.Utils.Database_NEW import Database


class SaveSensorDataCronjob(FixedTimeCronjob):
    def __init__(self):
        super(SaveSensorDataCronjob, self).__init__(task_name="SaveSensorDataCronjob")

    def task_to_do(self, *args):
        self.save_sensor_data()

    def get_seconds_to_wait(self, execution_time=None):
        t = datetime.datetime.today()

        seconds_to_wait = (60*60) - (t.minute*60) - t.second

        return seconds_to_wait

    def save_sensor_data(self):
        db = Database()

        sensors = []

        #Z-Wave Sensoren
        sensors.extend(ZWaveSensor.load_all(db))

        #MQTT Sensoren
        sensors.extend(MQTTSensor.load_all(db))

        for sensor in sensors:
            if sensor.save_data:
                self.save_to_db(sensor, db)

    def save_to_db(self, sensor, db: Database = None):
        if db is None:
            db = Database()
        time = datetime.datetime.now().strftime('%Y-%m-%d %H:00')

        db.insert("INSERT INTO SENSOR_DATA (DEVICE_ID, DEVICE_TYPE, DATETIME, VALUE) \
                        VALUES (:id, :type, :time, :value)",
                        {'id': sensor.id, 'type': sensor.get_device_type(), 'time': time, 'value': sensor.value})


import traceback

from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database


class HeatingSchemeItem(Item):
    def __init__(self, time, value, days, is_active, devices, id=None):
        super(HeatingSchemeItem, self).__init__()
        self.time = time
        self.value = value
        self.days = days
        self.is_active = is_active
        self.devices = devices
        self.id = id

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM HEATING_SCHEME WHERE ID == :id", {'id': self.id})
            db.delete("DELETE FROM HEATING_SCHEME_DAYS WHERE HEATING_SCHEME_ID == :id", {'id': self.id})
            db.delete("DELETE FROM HEATING_SCHEME_DEVICES WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                db.insert("""INSERT INTO EVENTS (TEXT, TYPE) VALUES (:text, :type)""",
                                {'text': self.text, 'type': self.type})
            # update
            else:
                db.update("""UPDATE EVENTS SET TEXT = :text, TYPE = :type WHERE ID = :id""",
                                {'text': self.text, 'type': self.type, 'id': self.id})
                # TODO add generated id to object
        except:
            if (Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save event to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'time': self.time,
            'value': self.value,
            'is_active': self.is_active,
            'days': self.days,
            'devices': self.devices
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = HeatingSchemeItem(result['TEXT'], result['TYPE'], result['TIMESTAMP'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database):
        if db is None:
            db = Database()
        return HeatingSchemeItem.load_all_from_db('SELECT * FROM EVENTS', {})

    @staticmethod
    def load_all_by_day(day, db: Database = None):
        if db is None:
            db = Database()
        return HeatingSchemeItem.load_all_from_db("""SELECT HEATING_SCHEME.ID, TIME, VALUE, ACTIVE, WEEKDAY_ID, 
        ROOMS.NAME as LOCATION, TYPE, DEVICE_ID FROM HEATING_SCHEME, HEATING_SCHEME_DAYS, HEATING_SCHEME_DEVICES, 
        ROOMS WHERE HEATING_SCHEME.ID = HEATING_SCHEME_DAYS.HEATING_SCHEME_ID AND HEATING_SCHEME.ID = 
        HEATING_SCHEME_DEVICES.ID AND HEATING_SCHEME_DEVICES.LOCATION = ROOMS.LOCATION AND WEEKDAY_ID = :day ORDER BY TIME""",
                                                  {'day': day})

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            time = dict['timestamp']
            value = dict['text']
            is_active = dict['is_active']
            days = dict['days']
            devices = dict['devices']

            item = HeatingSchemeItem(time, value, days, is_active, devices, id)

            return item
        except:
            raise InvalidParametersException("HeatingSchemeItem.create_from_dict(): invalid dict")


import select
import socket
import sys
import time

from Homevee.Helper import Logger

XBOX_PORT = 5050
XBOX_PING = "dd00000a000000000000000400000002"

py3 = sys.version_info[0] > 2


def xbox_wake_up(ip_address, live_id):
    Logger.log("Starting Xbox")

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.setblocking(0)
    s.bind(("", 0))
    s.connect((ip_address, XBOX_PORT))

    if isinstance(live_id, str):
        live_id = live_id.encode()
    else:
        live_id = live_id

    power_payload = b'\x00' + chr(len(live_id)).encode() + live_id + b'\x00'
    power_header = b'\xdd\x02\x00' + chr(len(power_payload)).encode() + b'\x00\x00'
    power_packet = power_header + power_payload
    Logger.log("Sending power on packets to {0}...".format(ip_address))
    send_power(s, power_packet)
    # ping_result = send_ping(s)

    s.close()


def send_power(s, data, times=5):
    for i in range(0, times):
        s.send(data)
        time.sleep(1)


def send_ping(s):
    s.send(bytearray.fromhex(XBOX_PING))
    return select.select([s], [], [], 5)[0]



#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules.CalendarModule import VoiceCalendarModule


class VoiceAddCalendarModule(VoiceCalendarModule):
    def get_context_key(self):
        return "VOICE_ADD_CALENDAR"

    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['erinnere', 'mich']
        ]

    def get_label(self):
        return "addcalendar"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.add_calendar(username, text, context, db)

    def add_calendar(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return {'msg_speech': 'Add Calendar', 'msg_text': 'Add Calendar'}


#!flask/bin/python
from flask import Flask

from Homevee.RestAPI.ProcessDataAPI import ProcessDataAPI

app = Flask(__name__)

#def update_cert(file):
    #cert_data = open(file).read()
    #cert_data = cert_data.replace("\n", "")
    #cert_data = cert_data.replace("-----BEGIN CERTIFICATE-----", "")
    #cert_data = cert_data.replace("-----END CERTIFICATE-----", "")

    #Database().do_query("REPLACE INTO CLOUD_CERTS (IP, CERTIFICATE) VALUES (%s, %s)",
    #                    (args.ip, cert_data))

def start_rest_api():
    # HOST = "test-cloud.homevee.de"
    # CERT_FILE = "/etc/letsencrypt/live/" + HOST + "/cert.pem"
    # CHAIN_FILE = "/etc/letsencrypt/live/" + HOST + "/chain.pem"
    # FULLCHAIN_FILE = "/etc/letsencrypt/live/" + HOST + "/fullchain.pem"
    # KEY_FILE = "/etc/letsencrypt/live/" + HOST + "/privkey.pem"

    blueprints = [ProcessDataAPI]
    for blueprint in blueprints:
        app.register_blueprint(blueprint)
    app.run(host='0.0.0.0', debug=True)
    # app.run(host=HOST, port=443, ssl_context=(FULLCHAIN_FILE, KEY_FILE))

if __name__ == '__main__':
    start_rest_api()


from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database


class ShoppingListItem(Item):
    def __init__(self, item, amount, id=None):
        super(ShoppingListItem, self).__init__()
        self.id = id
        self.item = item
        self.amount = amount

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (db is None):
                db = Database()
                db.delete("DELETE FROM SHOPPING_LIST WHERE ID == :id", {'id': self.id})
                return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if (db is None):
            db = Database()

        if(self.id is None or self.id == ""):
            #TODO Check if entry with same name exists and notify user(???)

            new_id = db.insert("INSERT OR IGNORE INTO SHOPPING_LIST (AMOUNT, ITEM) VALUES (:amount, :name);",
                        {'amount': self.amount, 'name': self.item})
            self.id = new_id
        else:
            db.update("UPDATE OR IGNORE SHOPPING_LIST SET AMOUNT = :amount, ITEM = :name WHERE ID = :id",
                        {'amount': self.amount, 'name': self.item, 'id': self.id})

    def build_dict(self):
        dict = {
            'id': self.id,
            'item': self.item,
            'amount': self.amount
        }
        return dict

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return ShoppingListItem.load_all_from_db('SELECT * FROM SHOPPING_LIST WHERE ID IN (%s)' % ','.join('?' * len(ids)),
                                     ids)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []

        results = db.select_all(query, params)

        for item in results:
            if item['AMOUNT'] is None:
                amount = -1
            else:
                amount = int(item['AMOUNT'])

            items.append(ShoppingListItem(item['ITEM'], amount, item['ID']))

        return items

    @staticmethod
    def load_all(db: Database):
        return ShoppingListItem.load_all_from_db('SELECT * FROM SHOPPING_LIST', {})

    @staticmethod
    def create_from_dict(dict):
        id = dict['id']
        item = dict['item']
        amount = dict['amount']

        return ShoppingListItem(item, amount, id)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from math import radians, cos, sqrt, atan2, sin

from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database

MIN_RETENTION_PERIOD = 5 * 60 * 1000 #5 minuten
MAX_DISTANCE = 500 #500 Meter

def update_gps(user, time, lat, lng, db: Database = None):
    if db is None:
        db = Database()

    on_location_change(user, int(time), float(lat), float(lng))

    db.insert("INSERT INTO GPS_DATA (user, TIMESTAMP, LATITUDE, LONGITUDE) VALUES (:user, :time, :lat, :lng);",
                {'user': user.username, 'time': time, 'lat': lat, 'lng': lng})

    return Status(type=STATUS_OK).get_dict()

def on_location_change(user, time, lat, lng, db: Database = None):
    if db is None:
        db = Database()
    #determine user location (at work, at school, at home, etc.)
    current_user_location = determine_user_location(user, time, lat, lng)
    set_user_location(user, current_user_location)

    return

def determine_user_location(user, time, lat, lng, db: Database = None):
    if db is None:
        db = Database()
    #if user is longer than MIN_RETENTION_PERIOD in roughly the same place => update his location status
    #check distance to place => less than MAX_DISTANCE => user is there

    place = get_nearest_place(user, lat, lng)

    location_data = db.select_all("SELECT * FROM GPS_DATA WHERE USERNAME = :user ORDER BY TIMESTAMP DESC LIMIT 50", {'user': user.username})
    if place is not None and place['DISTANCE'] <= MAX_DISTANCE:
        for location in location_data:
            if(compute_distance(place['LATITUDE'], place['LONGITUDE'], location['LATITUDE'], location['LONGITUDE']) <= MAX_DISTANCE):
                if(time - int(location['TIMESTAMP']) >= MIN_RETENTION_PERIOD):
                    return place
            else:
                break

    return None

def get_nearest_place(user, lat, lng, db: Database = None):
    if db is None:
        db = Database()

    place = db.select_one("SELECT *, ((LATITUDE - :lat)*(LATITUDE - :lat)) + ((LONGITUDE - :lng)*(LONGITUDE - :lng)) as DISTANCE FROM PLACES ORDER BY DISTANCE ASC LIMIT 1",
                {'lat': lat, 'lng': lng})

    return place

def compute_distance(lat1, lng1, lat2, lng2):
    R = 6373000.0

    lat1 = radians(float(lat1))
    lng1 = radians(float(lng1))
    lat2 = radians(float(lat2))
    lng2 = radians(float(lng2))

    dlon = lng2 - lng1
    dlat = lat2 - lat1

    a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
    c = 2 * atan2(sqrt(a), sqrt(1 - a))

    distance = R * c

    return distance

def set_user_location(user, place, db: Database = None):
    if db is None:
        db = Database()

    place_id = -1

    if(place is not None):
        place_id = place['ID']

    db.update("UPDATE USERDATA SET CURRENT_LOCATION = :loc WHERE USERNAME = :user",
                {'loc': place_id, 'user': user.username})

def get_gps_locations(user, db: Database = None):
    if db is None:
        db = Database()

    locations = []

    results = db.select_all("SELECT * FROM GPS_DATA WHERE USERNAME = :user", {'user': user.username})

    for item in results:
        if not is_near(locations, item['LATITUDE'], item['LONGITUDE']):
            locations.append({'time': item['TIMESTAMP'], 'latitude': item['LATITUDE'], 'longitude': item['LONGITUDE']})

    return {'locations': locations}

def is_near(locations, lat, lng):
    MIN_DISTANCE_TO_OTHERS = 1000

    for location in locations:
        if (compute_distance(location['latitude'], location['longitude'], lat, lng) < MIN_DISTANCE_TO_OTHERS):
            return True
    return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Functions.nutrition_data import get_nutrition_data
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule

MAX_PORTION_DEVIATION = 10

class VoiceGetNutritionInfoModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['wie', ['viel', 'viele'], ['fett', 'kalorien', 'kohlenhydrate', 'eiweiß', 'protein', 'zucker'], ['hat', 'ist', 'haben', 'sind']]
        ]

    def get_label(self):
        return "getnutritioninfo"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.voice_get_nutrition_info(username, text, context, db)

    def voice_get_nutrition_info(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        words = text.split(" ")

        number_words = ['ein', 'einer', 'eine', 'einem', 'einen']

        if 'in' in words:
            in_index = words.index('in')
            del words[in_index]

            index = None

            for number_word in number_words:
                if number_word in words:
                    index = words.index(number_word)

            if index is not None:
                del words[index]

        nutrition_types = ['fett', 'eiweiß', 'protein', 'kohlenhydrate', 'kalorien', 'zucker', 'proteine']

        nutrition_value = None

        for word in words:
            if word in nutrition_types:
                nutrition_value = word
                break

        if nutrition_value is None:
            return {'msg_speech': 'Es gab einen Fehler.',
                    'msg_text': 'Es gab einen Fehler.'}

        item = None
        trigger = ['ist', 'sind', 'haben', 'hat']
        add_word = False
        add_word_index = None
        amount_type = None
        amount = None
        for i in range(0, len(words)):
            word = words[i]

            if add_word_index is not None and i == add_word_index:
                add_word = True

            if add_word:
                if word in number_words:
                    continue

                if item is None:
                    item = word
                else:
                    item = item + " " + word
                continue

            if word in trigger:
                if 'gramm' in words:
                    index = words.index('gramm')
                    amount_type = 'gramm'
                elif 'kili' in words:
                    index = words.index('kilo')
                    amount_type = 'kilo'
                elif 'milliliter' in words:
                    index = words.index('milliliter')
                    amount_type = 'milliliter'
                elif 'liter' in words:
                    index = words.index('liter')
                    amount_type = 'liter'
                else:
                    add_word = True
                    continue

                amount = words[index - 1]
                add_word_index = index + 1

        answer = ""

        # Daten abfragen
        nutrition_data = get_nutrition_data(username, item, db)

        #print nutrition_data

        if nutrition_data is not None:
            if nutrition_data is not False:
                portionsize = nutrition_data['portionsize']

                if portionsize == 1:
                    portionsize_string = "Ein"
                else:
                    portionsize_string = str(portionsize)

                portion_string = portionsize_string + " " + nutrition_data['portionunit']

                name = nutrition_data['name']

                if (nutrition_value == "fett"):
                    value = nutrition_data['fat']
                    unit = "Gramm Fett"
                elif (nutrition_value in ['eiweiß', 'protein', 'proteine']):
                    value = nutrition_data['protein']
                    unit = "Gramm Eiweiß"
                elif (nutrition_value == "kohlenhydrate"):
                    value = nutrition_data['carbs']
                    unit = "Gramm Kohlenhydrate"
                elif (nutrition_value == "zucker"):
                    value = nutrition_data['sugar']
                    unit = "Gramm Zucker"
                elif (nutrition_value == "kalorien"):
                    value = nutrition_data['calories']
                    unit = "Kalorien"

                have_verb = "hat"

                if (portionsize > 1):
                    have_verb = "haben"

                answer = portion_string + " " + name + " " + have_verb + " " + str(value) + " " + unit + "."

                context_data = {'nutrition_type': nutrition_value}
                # set_context(username, 'nutrition', context_data, db)
            else:
                answer = "Es wurden keine Lebensmittel zum Suchbegriff "+item+" gefunden."

        else:
            #fehlermeldung ausgeben

            answer = "Die Nährwertdaten konnten nicht abgefragt werden."

        amount_text = ''
        if amount_type is not None and amount is not None:
            amount_text = ' => ' + amount + ' ' + amount_type

        return {'msg_speech': answer, 'msg_text': answer}


from Homevee.Item.Room import Room
from Homevee.Utils.Database_NEW import Database

def get_smart_speakers(user, db: Database = None):
    if db is None:
        db = Database()
    speakers =[]

    results = db.select_all("SELECT * FROM SMART_SPEAKER ORDER BY LOCATION ASC", {})

    for speaker in results:
        if user.has_permission(speaker['LOCATION']):
            item = {'name': speaker['NAME'], 'id': speaker['ID'], 'key': speaker['KEY'],
                    'location': speaker['LOCATION'], 'location_name': Room.get_name_by_id(speaker['LOCATION'],
                                                                                          db)}

            speakers.append(item)

    return {'speakers': speakers}

def add_edit_smart_speaker(username, db: Database = None):
    if db is None:
        db = Database()
    return


import traceback

from Homevee.Helper import translations, Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant import VoiceAssistant


def run_voice_tests():
    print("running voice tests...")

    commands = [
        "hallo",
        "mach das licht im garten an",
        "was steht an",
        "wie wird das wetter morgen",
        "was steht auf der einkaufsliste",
        "wie viel kalorien hat ein apfel",
        "was kommt im fernsehen",
        "was sind die tv tipps",
        "wer war paul walker",
        "wie lange brauche ich von velden nach münchen",
        "wie warm ist es in der küche",
        "was kann ich morgen mit freunden kostenlos machen",
        "erzähl mir einen witz",
        "was ist 5 mal 5",
        "wie ist der film titanic",
        "was ist der 24. dezember 2019 für ein tag",
        "was ist der 24. dezember für ein tag",
        "wie viel fett habe ich noch offen",
        "ksdjflksdfl",
    ]

    username = "sascha"
    db = Database()

    for command in commands:
        try:
            output = VoiceAssistant().do_voice_command(username, command, None, None, db, translations.LANGUAGE)

            output_text = output['msg_text']

            print("===> "+command+": "+output_text)
        except:
            print("error at command: "+command)
            if(Logger.IS_DEBUG):
                traceback.print_exc()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random
import traceback
import urllib.error
import urllib.parse
import urllib.request

from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetJokesModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['erzähl','sag'],'witz'],
            ['klopf', 'klopf']
        ]

    def get_label(self):
        return "joke"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_joke(username, text, context, db)

    def get_joke(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        try:
            url = Helper.SMART_API_PATH + "/?action=joke&text=" + urllib.parse.quote(text.encode('utf8'))
            Logger.log(url)
            data = urllib.request.urlopen(url).read()

            data = data.decode('utf-8')

        except Exception as e:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            data = None

        if data is not None:
            return {'msg_speech': data, 'msg_text': data}
        else:
            result = self.get_error()
            return {'msg_speech': result, 'msg_text': result}

    def get_error(self):
        return random.choice([
            'Mir fällt gerade kein Witz ein.'
        ])


#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import os

from Homevee.Utils.Constants import DATA_DIR


def get_image_directory_path(directory_name: str) -> str:
	"""
	Constructs the image directory path
	:param directory_name: the name of the directory
	:return: the image directory path
	"""
	path = os.path.join(DATA_DIR, "images", directory_name)
	return path

def create_image(filename: str, directory_name: str, img_data: str, optimize: bool = False) -> str:
	"""
	Creates an image
	:param filename: the filename of the image
	:param directory_name: the dirname of the image
	:param img_data: the img-data
	:param optimize: true if image should be optimized, false otherwise
	:return: the absolute path to the stored image
	"""
	filename = filename + ".jpeg"

	rel_path = os.path.join("images", directory_name)

	abs_path = os.path.join(DATA_DIR, rel_path)

	if not os.path.exists(abs_path):
		os.makedirs(abs_path)

	file_path = os.path.join(abs_path, filename)

	#Logger.log(img_data)

	img_data = base64.b64decode(img_data)

	fh = open(file_path, "wb")
	fh.write(img_data)
	fh.close()

	if(optimize):
		optimize_image(file_path)

	return os.path.join(rel_path, filename)

def optimize_image(file_path: str):
	"""
	Optimize the image
	:param file_path: path to the image
	:return:
	"""
	return


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from Homevee.Helper import Logger
from Homevee.Item.Gateway import *
from Homevee.Item.Status import *


def set_light_mode(user, id, mode, saturation, brightness, color, db, check_user=True):
    cur = db.cursor()

    db.select_all("SELECT LOCATION FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", {'id': id})

    result = cur.fetchone()

    if result is None:
        return Status(type=STATUS_ERROR).get_dict()
    elif check_user and not user.has_permission(result['LOCATION']):
        return {'result': 'nopermission'}

    gateway = Item.load_from_db(Gateway, PHILIPS_HUE_BRIDGE)

    if mode == True or mode == "1":
        mode = True
    else:
        mode = False

    data = {}

    db.select_all("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", {'id': id})

    result = cur.fetchone()

    if color is not None:
        rgb = tuple(int(color[i:i + 2], 16) for i in (0, 2, 4))
        x, y, z = rgb_to_xy(rgb)
        data['xy'] = [x, y]
        data['sat'] = int(z * 255)
    else:
        data['hue'] = result['HUE']

    if mode is not None:
        data['on'] = mode
    else:
        data['on'] = result['IS_ON'] == 1

    if brightness is not None:
        data['bri'] = int(float(brightness) * 255 / 100)
    else:
        data['bri'] = int(result['BRIGHTNESS'])

    Logger.log("rgb")

    response = requests.put("http://" + gateway.ip + "/api/" + gateway.key1 + "/lights/" + str(id) + "/state",
                            data=json.dumps(data))

    Logger.log(response.text)

    update_light_info(id, mode, color, saturation, brightness)

    return Status(type=STATUS_OK).get_dict()


def rgb_to_xy(rgb):
    red, green, blue = rgb

    if (red > 0.04045):
        red = pow((red + 0.055) / (1.0 + 0.055), 2.4)
    else:
        red = (red / 12.92)

    if (green > 0.04045):
        green = pow((green + 0.055) / (1.0 + 0.055), 2.4)
    else:
        green = (green / 12.92)

    if (blue > 0.04045):
        blue = pow((blue + 0.055) / (1.0 + 0.055), 2.4)
    else:
        blue = (blue / 12.92)

    X = red * 0.664511 + green * 0.154324 + blue * 0.162028
    Y = red * 0.283881 + green * 0.668433 + blue * 0.047685
    Z = red * 0.000088 + green * 0.072310 + blue * 0.986039

    if (X + Y + Z) == 0:
        fx = 0
        fy = 0
        fz = 0
    else:
        fx = X / (X + Y + Z)
        fy = Y / (X + Y + Z)
        fz = Z / (X + Y + Z)

    if (fx != fx):
        fx = 0.0

    if (fy != fy):
        fy = 0.0

    if (fz != fz):
        fz = 0.0

    return (fx, fy, fz)


def get_devices(db):
    gateway = Item.load_from_db(Gateway, PHILIPS_HUE_BRIDGE)

    try:
        response = requests.get("http://" + gateway.ip + "/api/" + gateway.key1 + "/lights").text

        Logger.log(response)

        data = json.loads(response)

        ids = list(data.keys())

        devices = []

        for id in ids:
            item = data[id]
            device_item = {'title': item['name'], 'id': id, 'room': None, 'type': 'light'}
            devices.append(device_item)
    except:
        Logger.log("Could not connect to Hue-Bridge...")

    return {'devices': devices}


def get_light_info(id, db: Database = None):
    if db is None:
        db = Database()


    gateway = Item.load_from_db(Gateway, PHILIPS_HUE_BRIDGE, db)

    try:
        request = requests.get("http://" + gateway.ip + "/api/" + gateway.key1 + "/lights/" + str(id))

        code = request.status_code

        if (code != 200):
            return False

        data = json.loads(request.text)

        mode = data['state']['on']
        brightness = int(float(data['state']['bri']) / 255 * 100)
        saturation = int(float(data['state']['sat']) / 255 * 100)
        hue = data['state']['hue']

        update_light_info(id, mode, hue, saturation, brightness, db)

        return True
    except:
        Logger.log("Could not connect to Hue-Bridge...")
        return False


def update_light_info(id, mode, hue, saturation, brightness, db: Database = None):
    if db is None:
        db = Database()
    if saturation is None:
        saturation = 255
    if brightness is None:
        brightness = 255
    db.update("""UPDATE PHILIPS_HUE_LIGHTS SET IS_ON = :mode, BRIGHTNESS = :bri, 
        SATURATION = :sat, HUE = :hue WHERE ID = :id""",
              {'mode': mode, 'bri': brightness, 'sat': saturation, 'hue': hue, 'id': id})



#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Cronjob import IntervalCronjob
from Homevee.Functions.Automations import Automation
from Homevee.Utils.Database_NEW import Database


class TimedAutomationsCronjob(IntervalCronjob, Automation):
    def __init__(self):
        super(TimedAutomationsCronjob, self).__init__(task_name="TimedAutomationsCronjob", interval_seconds=60)

    def task_to_do(self, *args):
        self.run_timed_automations()

    def run_timed_automations(self):
        # print "running automations"
        today = datetime.datetime.today()

        hour = today.hour
        minute = today.minute

        hour_string = str(hour)
        if (hour < 10):
            hour_string = "0" + hour_string

        minute_string = str(minute)
        if (minute < 10):
            minute_string = "0" + minute_string

        time_string = hour_string + ":" + minute_string

        # print time_string

        db = Database()

        automations = db.select_all(
            'SELECT * FROM AUTOMATION_DATA, AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_DATA.ID = AUTOMATION_RULE_ID AND TYPE = "time" AND IS_ACTIVE = "true" AND VALUE = :val',
            {'val': time_string})

        self.run_automations(automations, db)

        return


import json
import traceback

import requests

from Homevee.Helper import Logger
from Homevee.Helper.HomeveeCloud.Response import Response
from Homevee.Utils.Database_NEW import Database


class CloudAPI():
    def __init__(self, remote_id=None, access_token=None):
        db = Database()
        if remote_id is None:
            remote_id = db.get_server_data("REMOTE_ID")
        if access_token is None:
            access_token = db.get_server_data("REMOTE_ACCESS_TOKEN")
        self.remote_id = remote_id
        self.access_token = access_token
        self.url = "https://cloud.homevee.de"

    def do_post(self, path: str, params: dict, fields: list = None) -> Response:
        """
        Sends a post request to the CloudAPI
        :param path: the path to call
        :param params: the params
        :param fields: the fields
        :return: the response
        """
        try:
            body_data = json.dumps(params)
            headers = self.create_headers(body_data, fields)
            r = requests.post(self.url+path, data=body_data, headers=headers)
            return Response(r.status_code, r.content)
        except:
            traceback.print_exc()
            return None

    def do_get(self, path: str, params: dict, fields: list = None) -> Response:
        """
        Sends a get request to the CloudAPI
        :param path: the path to call
        :param params: the params
        :param fields: the fields
        :return: the response
        """
        try:
            body_data = ""
            headers = self.create_headers(body_data, fields)
            r = requests.get(self.url+path, data=body_data, headers=headers)
            return Response(r.status_code, r.content)
        except:
            #traceback.print_exc()
            return None

    def do_delete(self, path: str, params: dict, fields: list = None) -> Response:
        """
        Sends a delete request to the CloudAPI
        :param path: the path to call
        :param params: the params
        :param fields: the fields
        :return: the response
        """
        try:
            body_data = json.dumps(params)
            headers = self.create_headers(body_data, fields)
            r = requests.delete(self.url+path, data=body_data, headers=headers)
            return Response(r.status_code, r.content)
        except:
            #traceback.print_exc()
            return None

    def do_put(self, path: str, params: dict, fields: list = None) -> Response:
        """
        Sends a put request to the CloudAPI
        :param path: the path to call
        :param params: the params
        :param fields: the fields
        :return: the response
        """
        try:
            body_data = json.dumps(params)
            headers = self.create_headers(body_data, fields)
            r = requests.put(self.url+path, data=body_data, headers=headers)
            return Response(r.status_code, r.content)
        except:
            #traceback.print_exc()
            return None

    def create_headers(self, body: str, fields: list = None) -> dict:
        """
        Creates the headers with the given fields
        :param body: the body
        :param fields: the fields of the header
        :return: the header dict
        """
        headers = {
            #"Host": self.url,
            "Connection": "keep-alive",
            "Content-Length": bytes(len(body)),
            "X-Requested-With": "XMLHttpRequest",
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.52 Safari/536.5",
            "Content-Type": "application/json",
            #"Accept": "*/*",
            #"Referer": "https://www.mywbsite.fr/data/mult.aspx",
            #"Accept-Encoding": "gzip,deflate,sdch",
            #"Accept-Language": "fr-FR,fr;q=0.8,en-US;q=0.6,en;q=0.4",
            #"Accept-Charset": "ISO-8859-1,utf-8;q=0.7,*;q=0.3",
            "Remote-ID": self.remote_id,
            "Access-Token": self.access_token
        }
        if fields is not None:
            for key in fields:
                headers[key] = fields[key]
        if Logger.IS_DEBUG:
            print(headers)
        return headers


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request

from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetActivitiesModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['mir', 'uns'], 'ist', 'langweilig'],
            ['was', ['kann', 'können'], ['ich', 'wir'], ['tun', 'machen', 'unternehmen']]
        ]

    def get_label(self):
        return "activity"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_activities(username, text, context, db)

    def get_activities(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        try:
            url = Helper.SMART_API_PATH + "/?action=activity&text=" + urllib.parse.quote(text.encode('utf8'))
            Logger.log(url)

            response = urllib.request.urlopen(url)
            data = response.read()

            data = data.decode('utf-8')

            if data is not None:
                return {'msg_speech': data, 'msg_text': data}
            else:
                output = self.get_error()
        except urllib.error.HTTPError as e:
            output = self.get_error()

        return {'msg_speech': output, 'msg_text': output}

    def get_error(self):
        return random.choice([
            'Mir fällt gerade nichts ein, was du tun könntest.'
        ])


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Device import Device
from Homevee.Item.Person import Person
from Homevee.Utils.Database_NEW import Database


def add_edit_person(user, id, name, nickname, address, latitude, longitude, phonenumber, birthdate, db: Database = None):
    if db is None:
        db = Database()
    try:
        person = Device.load_from_db(Person, id, db)
        person.name = name
        person.nickname = nickname
        person.phone_number = phonenumber
        person.address = address
        person.birthdate = birthdate
        person.longitude = longitude
        person.latitude = latitude
    except:
        person = Person(name, nickname, phonenumber, address, birthdate, longitude, latitude)

    return person.api_save()

def get_persons(db):
    persons = Person.load_all(db)
    return {'persons': Person.list_to_dict(persons)}

def delete_person(user, id, db: Database = None):
    if db is None:
        db = Database()
    person = Device.load_from_db(Person, id)
    return person.api_delete()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule

class VoiceCalculatorModule(VoiceModule):
    def calculator(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return {'msg_speech':"Taschenrechner", 'msg_text':"Taschenrechner"}

    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['wie', 'viel',['ist','ergibt']],
            ['was',['ist','ergibt']]
        ]

    def get_label(self):
        return "calculator"

    def get_context_key(self):
        return "CONTEXT_CALCULATOR"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.calculator(username, text, context, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import io
import json
import os
import time
import traceback
from _thread import start_new_thread

from PIL import Image

from Homevee.Functions.tensorflow_functions import people_predict, people_retrain
from Homevee.Helper import Logger
from Homevee.Item.Status import *
from Homevee.Utils import Constants
from Homevee.Utils.Constants import DATA_DIR
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.FileUtils import create_image

IS_TRAINING_TAG = "PEOPLE_CLASSIFIER_TRAINING_RUNNING"

def upload_images(user, data, img_class, db: Database = None):
    if db is None:
        db = Database()
    images = json.loads(data)

    counter = 0

    if(img_class == "-1" or img_class == -1):
        img_class = None

    for image in images:
        filename = "person-" + str(int(time.time())) + "_" + str(counter)

        counter += 1

        image_path = create_image(filename, "people", image, optimize=True)

        db.insert("INSERT INTO PEOPLE_LEARNING_DATA (PATH, PERSON_ID) VALUES (:path, :person_id)",
                    {'path': image_path, 'person_id': img_class})

        #print image

    return Status(type=STATUS_OK).get_dict()

def save_people_class(user, id, data, classname, db: Database = None):
    if db is None:
        db = Database()
    return

def classify_person(image_data, db: Database = None):
    if db is None:
        db = Database()
    filename = "person-"+str(int(time.time()))

    image_path = create_image(filename, "people", image_data, optimize=True)

    db.insert("INSERT INTO PEOPLE_LEARNING_DATA (PATH) VALUES (:path)",
                {'path': image_path})

    predictions = people_predict.predict(os.path.join(Constants.DATA_DIR, image_path))

    for prediction in predictions:
        person_data = get_person_class_data(prediction['prediction'])
        Logger.log(person_data['NAME'] + ": " + str(prediction['confidence']))

    Logger.log(image_path)

    person_data = get_person_class_data(predictions[0]['prediction'])

    return {'label': person_data['NAME'], 'confidence': str(predictions[0]['confidence']*100)}

def get_person_class_data(id, db: Database = None):
    if db is None:
        db = Database()

    return db.select_one("SELECT * FROM PEOPLE_DATA WHERE ID = :id;", {'id': id})

def start_people_training(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return {'result': 'noadmin'}

    is_training = db.get_server_data(IS_TRAINING_TAG)

    if(is_training is not None and is_training == "true"):
        return {'result': 'alreadytraining'}

    start_new_thread(training_thread, (user, None))

    return Status(type=STATUS_OK).get_dict()

def training_thread(user, d):
    db = Database()

    db.set_server_data(IS_TRAINING_TAG, "true")

    try:
        people_retrain.people_training(user.username)
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

    db.set_server_data(IS_TRAINING_TAG, "false")

def get_people_classes(db):
        results = db.select_all("SELECT * FROM PEOPLE_DATA")

        classes = []

        for row in results:
            item = {'id': int(row['ID']), 'name': row['NAME'],}
            classes.append(item)

        item = {'id': None, 'name': 'Nicht zugeordnet'}
        classes.append(item)

        return {'classes': classes}

def get_people_class_images(image_class, show, offset, db: Database = None):
    if db is None:
        db = Database()
    param_array = {'limit': show, 'offset': offset}

    if image_class is None or image_class == "-1" or image_class == -1:
        results = db.select_all("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID IS NULL OR PERSON_ID = -1 LIMIT :limit OFFSET :offset",
                    param_array)
    else:
        param_array['key'] = image_class
        results = db.select_all("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID = :key LIMIT :limit OFFSET :offset",
                    param_array)

    images = []

    for img in results:
        images.append(img['ID'])

    return {'images': images}

def delete_people_images(ids, db: Database = None):
    if db is None:
        db = Database()
    # todo

    ids = json.loads(ids)

    for id in ids:
        Logger.log("deleting: "+str(id))
        item = db.select_one("SELECT * FROM PEOPLE_LEARNING_DATA WHERE ID == :id;",
                    {'id': id})

        if (item is not None):
            rel_path = item['PATH']

            image_path = os.path.join(DATA_DIR, rel_path)
            if os.path.exists(image_path):
                os.remove(image_path)
            else:
                Logger.log(image_path+" not found")

            db.delete("DELETE FROM PEOPLE_LEARNING_DATA WHERE ID == :id;",
                        {'id': id})

    return Status(type=STATUS_OK).get_dict()

def change_people_image_class(ids, newclass, db: Database = None):
    if db is None:
        db = Database()
    if newclass == "-1":
        newclass = None

    ids = json.loads(ids)

    Logger.log(ids)

    for id in ids:
        db.update("UPDATE 'PEOPLE_LEARNING_DATA' SET PERSON_ID = :key WHERE ID == :id;",
                    {'key': newclass, 'id': id})

    return Status(type=STATUS_OK).get_dict()

def get_people_images(image_class, show, offset, db: Database = None):
    if db is None:
        db = Database()
    param_array = {'limit': show, 'offset': offset}

    if image_class == "-1":
        results = db.select_all("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID IS NULL LIMIT :limit OFFSET :offset",
                    param_array)
    else:
        param_array['key'] = image_class
        results = db.select_all("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID = :key LIMIT :limit OFFSET :offset",
                    param_array)

    images = []

    for img in results:
        images.append(img['ID'])

    return {'images': images}

def get_people_image(id, db: Database = None):
    if db is None:
        db = Database()
    result = db.select_all("SELECT PATH FROM 'PEOPLE_LEARNING_DATA' WHERE ID == :id",
                {'id': id})

    rel_path = result['PATH']

    path = os.path.join(DATA_DIR, rel_path)
    Logger.log(("Path: " + path))

    try:
        im = Image.open(path)
    except:
        return {'imagedata': None}

    size = 50

    image_dimensions = size, size

    im.thumbnail(image_dimensions, Image.ANTIALIAS)

    buffer = io.BytesIO()
    im.save(buffer, format="JPEG")
    encoded_string = base64.b64encode(buffer.getvalue())
    encoded_string = encoded_string.decode('utf-8')
    im.close()

    #with open(path, "rb") as image_file:
    #    encoded_string = base64.b64encode(image_file.read())

    return {'imagedata': encoded_string}

def get_performance_settings(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    prefix = "PEOPLE_"

    tf_data = {}

    indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

    for index in indices:
        result = db.select_one("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                    {'key': prefix+index})

        if(result is not None and 'VALUE' in result):
            tf_data[index] = result['VALUE']

    return {'data': tf_data, 'graph_sizes': people_retrain.get_graph_sizes(),
            'image_sizes' : people_retrain.get_image_sizes()}

def set_performance_settings(user, data, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return {'result': 'noadmin'}

    prefix = "PEOPLE_"

    data = json.loads(data)

    indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

    for index in indices:
        param_array = {'value': data[index], 'key': prefix+index}

        db.update("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key;", param_array)

        db.insert("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES (:value, :key);", param_array)

    return {'result':'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-

import time

from Homevee.Item.Event import Event
from Homevee.Utils.Database_NEW import Database

'''Gibt die Ereignisse zurück'''
def get_events(user, type, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    events = Event.load_all_from_db_desc_date_by_type(offset, limit, type)
    user.events_last_checked = time.time()
    user.save_to_db(db)
    return {'events': Event.list_to_dict(events)}

'''Gibt die Anzahl der ungesehenen Ereignisse zurück'''
def get_unseen_events(user, db: Database = None):
    if db is None:
        db = Database()
    count = Event.get_unseen_events(user)
    return {'eventcount': count}

'''Gibt die vorhandenen Ereignis-Typen zurück'''
def get_event_types(db):
    results = db.select_all("SELECT DISTINCT TYPE FROM EVENTS ORDER BY TYPE", {})

    types = []

    for data in results:
        types.append(data['TYPE'])

    return {'types': types}

'''Erstellt ein neues Ereignis'''
def add_event(type, text, db: Database = None):
    if db is None:
        db = Database()
    event = Event(text=text, type=type)
    return event.save_to_db()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os

from Homevee.DeviceAPI import max_cube, zwave, philips_hue, rademacher_homepilot
from Homevee.Functions.room_data import get_rooms
from Homevee.Functions.sensor_data import SENSOR_TYPE_MAP
from Homevee.Helper import Logger
from Homevee.Item.Gateway import *
from Homevee.Item.Status import *
from Homevee.Utils.DeviceTypes import *


def get_gateway_devices(user, type, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    if type == Z_WAVE_GATEWAY:
        return zwave.get_devices.get_devices(db)
    if type == PHILIPS_HUE_BRIDGE:
        return philips_hue.get_devices(db)
    if type == RADEMACHER_HOMEPILOT:
        return rademacher_homepilot.get_devices(db)
    elif type == MAX_CUBE:
        gateway = Item.load_from_db(Gateway, MAX_CUBE, db)

        Logger.log(gateway.ip)

        devices = max_cube.get_devices(gateway.ip)

        if devices is not None:
            return {'devices': devices}
        else:
            return {'devices': []}
    elif type == "Network":
        network_devices = []

        hostnames = os.popen("arp -a").read()

        ip_adresses = os.popen("arp -a").read()

        Logger.log(hostnames)
        #print ip_adresses

        #for i in range(0, len(hostnames)):

        return {'devices': network_devices}
    else:
        return "nosuchgateway"

'''
function getGatewayDevices($type, $db){
	switch($type){
		case "Network":
			$networkDevices = array();

			exec("arp", $hostnames);
			exec("arp -n", $ipAdresses);

			for($i = 1; $i < sizeOf($hostnames); $i++){

				$names = preg_replace('!\s+!', ' ', $hostnames[$i]);

				$name = explode(" ", $names)[0];

				$adresses = preg_replace('!\s+!', ' ', $ipAdresses[$i]);

				$data = explode(" ", $adresses);

				$ip = $data[0];

				$mac = $data[2];

				array_push($networkDevices, array('id' => $ip, 'title' => $name, 'id2' => $mac));
			}

			return array('devices' => $networkDevices);
		default:
			return "nosuchgateway";
	}
}
'''

def add_edit_device(user, device_id, type, data, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    add_new = (device_id == None or device_id == "" or device_id == "-1")

    type = type.decode("utf-8")

    data_array = json.loads(data)



    #XBOX One WOL
    if type == XBOX_ONE_WOL:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'],
                       'ip': data_array['ip'], 'live_id': data_array['liveid']}

        if device_id == "-1" or device_id is -1:
            db.insert(
                "INSERT OR IGNORE INTO 'XBOX_ONE_WOL' (LOCATION, ICON, NAME, IP_ADDRESS, XBOX_LIVE_ID) VALUES (:room, :icon, :name, :ip, :live_id);",
                param_array)
        else:

            param_array['id'] = device_id

            db.update(
                "UPDATE OR IGNORE 'XBOX_ONE_WOL' SET LOCATION = :room, ICON = :icon, NAME = :name, IP_ADDRESS = :ip, XBOX_LIVE_ID = :live_id WHERE ID == :id;",
                param_array)

    # Philips Hue
    elif type == PHILIPS_HUE_LIGHT:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

        db.insert(
            "INSERT OR IGNORE INTO 'PHILIPS_HUE_LIGHTS' (LOCATION, ID, ICON, NAME) VALUES (:room, :id, :icon, :name);",
            param_array)

        db.update(
            "UPDATE OR IGNORE 'PHILIPS_HUE_LIGHTS' SET LOCATION = :room, ICON = :icon, NAME = :name WHERE ID == :id;",
            param_array)
    #MQTT Sensor
    elif type == MQTT_SENSOR:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data['icon'], 'id': device_id,
                       'key': data_array['key'], 'topic': data_array['topic'], 'save_data': data_array['save_data']}

        db.insert(
            "INSERT OR IGNORE INTO 'MQTT_SENSORS' (ROOM, ID, ICON, NAME, KEY, TOPIC, SAVE_DATA) VALUES (:room, :id, :icon, :name, :key, :topic, :save_data);",
            param_array)

        db.update(
            "UPDATE OR IGNORE 'MQTT_SENSORS' SET ROOM = :room, ICON = :icon, NAME = :name, KEY = :key, TOPIC = :topic, SAVE_DATA = :save_data, WHERE ID == :id;",
            param_array)

    #433 MHz
    elif type == FUNKSTECKDOSE:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'],
                       'hauscode': data_array['hauscode'], 'nummer': data_array['steckdosennummer']}

        '''if add_new:
                        data = db.select_one("SELECT COUNT(*) FROM 'funksteckdosen' WHERE HAUSCODE = :hauscode AND STECKDOSENNUMMER = :nummer;",
                                    {'hauscode': data_array['hauscode'], 'nummer': data_array['steckdosennummer']})

                        count = cur.fetchone()['COUNT(*)']

                        if(int(count) != 0):
                            return "hauscodesocketnumberinuse"
                    else:
                        param_array['id'] = device_id'''

        if add_new:
            db.insert("INSERT INTO 'funksteckdosen' (ROOM, HAUSCODE, STECKDOSENNUMMER, ICON, NAME) VALUES (:room, :hauscode, :nummer, :icon, :name);",
                        param_array)
        else:
            param_array['id'] = device_id

            db.update("UPDATE 'funksteckdosen' SET ROOM = :room, HAUSCODE = :hauscode, STECKDOSENNUMMER = :nummer, ICON = :icon, NAME = :name WHERE DEVICE == :id;",
                        param_array)
    # MAX!
    elif type == MAX_THERMOSTAT:
        param_array = {'room': data_array['room'], 'name': data_array['name'],
                       'icon': data_array['icon'], 'id': device_id}

        db.update("UPDATE OR IGNORE 'MAX_THERMOSTATS' SET RAUM = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
            param_array)

        db.insert("INSERT OR IGNORE INTO 'MAX_THERMOSTATS' (RAUM, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
            param_array)
    # Rademacher!
    elif type == RADEMACHER_THERMOSTAT:
        param_array = {'room': data_array['room'], 'name': data_array['name'],
                       'icon': data_array['icon'], 'id': device_id}

        db.update("UPDATE OR IGNORE 'HOMEPILOT_THERMOSTATS' SET ROOM = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
            param_array)

        db.insert("INSERT OR IGNORE INTO 'HOMEPILOT_THERMOSTATS' (ROOM, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
            param_array)
    elif type == RADEMACHER_BLIND_CONTROL:
        param_array = {'room': data_array['room'], 'name': data_array['name'],
                       'icon': data_array['icon'], 'id': device_id}

        db.update("UPDATE OR IGNORE 'HOMEPILOT_BLIND_CONTROL' SET LOCATION = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
            param_array)

        db.insert("INSERT OR IGNORE INTO 'HOMEPILOT_BLIND_CONTROL' (LOCATION, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
            param_array)

    #Z-Wave
    elif type == ZWAVE_SENSOR:
        param_array = {'room': data_array['room'], 'savedata': data_array['savedata'], 'shortform': data_array['name'],
                       'sensor_type': data_array['sensor_type'], 'icon': data_array['icon'], 'id': device_id}

        count = db.select_one("SELECT COUNT(*) FROM ZWAVE_SENSOREN WHERE ID = :id", {'id': device_id})
        add_new = (count['COUNT(*)'] == 0)

        if add_new:
            db.insert("INSERT INTO 'ZWAVE_SENSOREN' (RAUM, ID, SHORTFORM, ICON, SAVE_DATA, SENSOR_TYPE) VALUES (:room, :id, :shortform, :icon, :savedata, :sensor_type);",
                        param_array)
        else:
            db.update("UPDATE 'ZWAVE_SENSOREN' SET RAUM = :room, SHORTFORM = :shortform, ICON = :icon, SAVE_DATA = :savedata, SENSOR_TYPE = :sensor_type WHERE ID = :id;",
                        param_array)
    elif type == ZWAVE_THERMOSTAT:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

        count = db.select_one("SELECT COUNT(*) FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", {'id': device_id})
        add_new = (count['COUNT(*)'] == 0)

        if add_new:
            db.insert("INSERT INTO 'ZWAVE_THERMOSTATS' (RAUM, NAME, THERMOSTAT_ID, ICON) VALUES (:room,:name,:id,:icon);",
            param_array)
        else:
            db.update("UPDATE 'ZWAVE_THERMOSTATS' SET RAUM = :room, NAME = :name, THERMOSTAT_ID = :id, ICON =:icon WHERE THERMOSTAT_ID = :id;",
            param_array)
    elif type == ZWAVE_POWER_METER:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id, 'daily_reset': data_array['is_reset_daily']}

        count = db.select_one("SELECT COUNT(*) FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", {'id': device_id})
        add_new = (count['COUNT(*)'] == 0)

        if add_new:
            db.insert("INSERT INTO 'ZWAVE_POWER_METER' (ROOM_ID, DEVICE_ID, DEVICE_NAME, ICON, IS_RESET_DAILY) VALUES (:room,:id,:name,:icon,:daily_reset);",
            param_array)
        else:
            db.update("UPDATE 'ZWAVE_POWER_METER' SET ROOM_ID = :room, DEVICE_NAME = :name, DEVICE_ID = :id, ICON =:icon, IS_RESET_DAILY = :daily_reset WHERE DEVICE_ID =:id;",
            param_array)
    elif type == ZWAVE_SWITCH:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

        count = db.select_one("SELECT COUNT(*) FROM ZWAVE_SWITCHES WHERE ID = :id", {'id': device_id})
        add_new = (count['COUNT(*)'] == 0)

        if add_new:
            db.insert("INSERT INTO 'ZWAVE_SWITCHES' (LOCATION, NAME, ID, ICON) VALUES (:room,:name,:id,:icon);",
            param_array)
        else:
            db.update("UPDATE 'ZWAVE_SWITCHES' SET LOCATION = :room, NAME = :name, ID = :id, ICON =:icon WHERE ID =:id;",
            param_array)
    elif type == ZWAVE_DIMMER:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

        count = db.select_one("SELECT COUNT(*) FROM ZWAVE_DIMMER WHERE ID = :id", {'id': device_id})
        add_new = (count['COUNT(*)'] == 0)

        if add_new:
            db.insert("INSERT INTO 'ZWAVE_DIMMER' (LOCATION, NAME, ID, ICON) VALUES (:room,:name,:id,:icon);",
            param_array)
        else:
            db.update("UPDATE 'ZWAVE_DIMMER' SET LOCATION = :room, NAME = :name, ID = :id, ICON =:icon WHERE ID =:id;",
            param_array)
    #Media Center
    elif type == MEDIA_CENTER:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'ip': data_array['ip'],
                       'username': data_array['username'], 'password': data_array['password'], 'type': data_array['type']}

        if add_new:
            db.insert("INSERT INTO 'MEDIA_CENTER' (LOCATION, NAME, IP, USERNAME, PASSWORD, TYPE) VALUES (:room,:name, :ip, :username, :password, :type);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'MEDIA_CENTER' SET LOCATION = :room, NAME = :name, IP = :id, USERNAME = :username, PASSWORD = :password, TYPE = :type, ICON =:icon WHERE ID = :id;",
            param_array)
    #URL-Devices
    elif type == URL_SWITCH:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'on': data_array['onurl'],
                       'off': data_array['offurl'], 'val': data_array['valurl'], 'icon': data_array['icon']}

        if add_new:
            db.insert("INSERT INTO 'URL_SWITCH_BINARY' (LOCATION, NAME, URL_ON, URL_OFF, URL_GET_STATE, ICON) VALUES (:room,:name, :on, :off, :val, :icon);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'URL_SWITCH_BINARY' SET LOCATION = :room, NAME = :name, URL_ON = :on, URL_OFF = :off, URL_GET_STATE = :val, ICON =:icon WHERE ID = :id;",
            param_array)
    elif type == URL_TOGGLE:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'toggle': data_array['toggleurl'],
                       'icon': data_array['icon']}

        if add_new:
            db.insert("INSERT INTO 'URL_TOGGLE' (LOCATION, NAME, TOGGLE_URL, ICON) VALUES (:room, :name, :toggle, :icon);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'URL_TOGGLE' SET LOCATION = :room, NAME = :name, TOGGLE_URL = :toggle, ICON =:icon WHERE ID = :id;",
            param_array)
    elif type == URL_RGB_LIGHT:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'url': data_array['url'],
                       'icon': data_array['icon']}

        if add_new:
            db.insert("INSERT INTO 'URL_RGB_LIGHT' (LOCATION, NAME, URL, ICON) VALUES (:room, :name, :url, :icon);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'URL_RGB_LIGHT' SET LOCATION = :room, NAME = :name, URL = :toggle, ICON =:icon WHERE ID = :id;",
            param_array)
    #MQTT
    elif type == MQTT_TRIGGER:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'topic': data_array['topic'],
                       'icon': data_array['icon'], 'type': data_array['type']}

        if add_new:
            db.insert("INSERT INTO 'MQTT_TRIGGERS' (LOCATION, NAME, TOPIC, TYPE, ICON) VALUES (:room, :name, :topic, :type, :icon);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'MQTT_TRIGGERS' SET LOCATION = :room, NAME = :name, TOPIC = :topic, ICON =:icon WHERE ID = :id;",
            param_array)
    elif type == MQTT_SENSOR:
        param_array = {'room': data_array['room'], 'name': data_array['name'], 'topic': data_array['topic'],
                       'icon': data_array['icon'], 'type': data_array['type']}

        if add_new:
            db.insert("INSERT INTO 'MQTT_SENSORS' (LOCATION, NAME, TOPIC, TYPE, ICON) VALUES (:room, :name, :topic, :type, :icon);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'MQTT_SENSORS' SET LOCATION = :room, NAME = :name, TOPIC = :topic, ICON =:icon WHERE ID = :id;",
            param_array)
    #Wake on Lan
    elif type == WAKE_ON_LAN:
        param_array = {'room': data_array['room'], 'name': data_array['name'],
                       'icon': data_array['icon'], 'mac': data_array['mac']}

        if add_new:
            db.insert("INSERT INTO 'WAKE_ON_LAN' (LOCATION, NAME, ICON, MAC_ADDRESS) VALUES (:room, :name, :icon, :mac);",
            param_array)
        else:
            param_array['id'] = device_id
            db.update("UPDATE 'WAKE_ON_LAN' SET LOCATION = :room, NAME = :name, MAC_ADDRESS = :mac, ICON =:icon WHERE DEVICE = :id;",
            param_array)
    else:
        return {'result': 'nosuchtype'}

    return Status(type=STATUS_OK).get_dict()

def delete_device(user, type, id, db: Database = None):
    if db is None:
        db = Database()
    #type = type.encode('utf-8')
    db_cols = {
        FUNKSTECKDOSE: {'table': 'FUNKSTECKDOSEN', 'location': 'ROOM', 'id': 'DEVICE'},
        PHILIPS_HUE_LIGHT: {'table': 'PHILIPS_HUE_LIGHTS', 'location': 'LOCATION', 'id': 'ID'},
        WAKE_ON_LAN: {'table': 'WAKE_ON_LAN', 'location': 'LOCATION', 'id': 'DEVICE'},
        XBOX_ONE_WOL: {'table': 'XBOX_ONE_WOL', 'location': 'LOCATION', 'id': 'ID'},
        URL_RGB_LIGHT: {'table': 'URL_RGB_LIGHT', 'location': 'LOCATION', 'id': 'ID'},
        MQTT_SENSOR: {'table': 'MQTT_SENSORS', 'location': 'ROOM', 'id': 'ID'},
        URL_TOGGLE: {'table': 'URL_TOGGLE', 'location': 'LOCATION', 'id': 'ID'},
        URL_SWITCH: {'table': 'URL_SWITCH', 'location': 'LOCATION', 'id': 'ID'},
        ZWAVE_DIMMER: {'table': 'ZWAVE_DIMMER', 'location': 'LOCATION', 'id': 'ID'},
        ZWAVE_SWITCH: {'table': 'ZWAVE_SWITCHES', 'location': 'LOCATION', 'id': 'ID'},
        ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'location': 'RAUM', 'id': 'ID'},
        ZWAVE_POWER_METER: {'table': 'ZWAVE_POWER_METER', 'location': 'ROOM_ID', 'id': 'DEVICE_ID'},
        ZWAVE_THERMOSTAT: {'table': 'ZWAVE_THERMOSTATS', 'location': 'RAUM', 'id': 'THERMOSTAT_ID'},
        MAX_THERMOSTAT: {'table': 'MAX_THERMOSTATS', 'location': 'RAUM', 'id': 'ID'},
        RADEMACHER_THERMOSTAT: {'table': 'HOMEPILOT_THERMOSTATS', 'location': 'ROOM', 'id': 'ID'},
        RADEMACHER_BLIND_CONTROL: {'table': 'HOMEPILOT_BLIND_CONTROL', 'location': 'LOCATION', 'id': 'ID'}
    }

    if type not in db_cols:
        return {'result': 'nosuchtype'}

    data = db.select_one("SELECT * FROM "+db_cols[type]['table']+" WHERE "+db_cols[type]['id']+" = :id",
                {'id': id})

    Logger.log(data)

    if not user.has_permission(data[db_cols[type]['location']]):
        return {'result': 'nopermission'}

    db.delete("DELETE FROM "+db_cols[type]['table']+" WHERE " + db_cols[type]['id'] + " = :id",
                {'id': id})

    return Status(type=STATUS_OK).get_dict()

def get_device_data(user, type, id, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return {'result': 'noadmin'}

    room_array = {}

    for room in get_rooms(user, db)['rooms']:
        room_array[room['location']] = room['name']

    Logger.log(room_array)

    param_array = {'id': id}

    device_data = {}

    #Funksteckdose
    if type == FUNKSTECKDOSE:
        data = db.select_one("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE = :id", param_array)

        device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'],
                       'hauscode': data['HAUSCODE'], 'icon': data['ICON'], 'steckdosennummer': data['STECKDOSENNUMMER']}
    #Z-Wave
    elif type == ZWAVE_POWER_METER:
        data = db.select_one("SELECT * FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", param_array)

        device_data = {'room': room_array[int(data['ROOM_ID'])], 'location': data['ROOM_ID'], 'name': data['DEVICE_NAME'],
                       'is_reset_daily': (data['IS_RESET_DAILY']==1), 'icon': data['ICON']}
    elif type == ZWAVE_SENSOR:
        data = db.select_one("SELECT * FROM ZWAVE_SENSOREN WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['SHORTFORM'],
                       'save_data': data['SAVE_DATA'], 'icon': data['ICON'],
                       'sensor_type': data['SENSOR_TYPE'], 'sensor_type_name': SENSOR_TYPE_MAP[data['SENSOR_TYPE']]['name']}
    elif type == ZWAVE_THERMOSTAT:
        data = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", param_array)

        device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['NAME'],
                       'icon': data['ICON']}
    elif type == ZWAVE_SWITCH:
        data = db.select_one("SELECT * FROM ZWAVE_SWITCHES WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON']}
    elif type == ZWAVE_DIMMER:
        data = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON']}
    #MQTT-Sensor
    elif type == MQTT_SENSOR:
        data = db.select_one("SELECT * FROM MQTT_SENSORS, MQTT_DEVICES WHERE MQTT_SENSORS.DEVICE_ID = MQTT_DEVICES.ID AND MQTT_SENSORS.ID = :id", param_array)

        device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'], 'icon': data['ICON'],
                       'topic': data['TOPIC'], 'key': data['KEY'], 'sensor_type': data['TYPE'],
                       'sensor_type_name': SENSOR_TYPE_MAP[data['TYPE']]['name'], 'save_data': data['SAVE_DATA']==1}
    #IP Kamera
    elif type == IP_CAM:
        data = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'ip': data['IP'], 'port': data['PORT'], 'path': data['PATH'],
                       'username': data['USERNAME'], 'password': data['PASSWORD'], 'recordfootage': data['RECORD_FOOTAGE'],
                       'autodelete': data['AUTO_DELETE'], 'detectionthreshold': data['MOTION_DETECTION_THRESHOLD']}
    #URL-Toggle
    elif type == URL_TOGGLE:
        data = db.select_one("SELECT * FROM URL_TOGGLE WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'toggle_url': data['TOGGLE_URL']}
    #URL-Switch
    elif type == URL_SWITCH:
        data = db.select_one("SELECT * FROM URL_SWITCH_BINARY WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'on_url': data['URL_ON'], 'off_url': data['URL_OFF'],
                       'get_state_url': data['URL_GET_STATE']}
    #URL RGB-Licht
    elif type == URL_RGB_LIGHT:
        data = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'url': data['URL']}
    #Philips Hue RGB-Licht
    elif type == PHILIPS_HUE_LIGHT:
        data = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'type': data['TYPE']}
    #MAX Thermostat
    elif type == MAX_THERMOSTAT:
        data = db.select_one("SELECT * FROM MAX_THERMOSTATS WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['NAME'],
                       'icon': data['ICON']}
    #Wake on Lan
    elif type == WAKE_ON_LAN:
        data = db.select_one("SELECT * FROM WAKE_ON_LAN WHERE DEVICE = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'mac': data['MAC_ADDRESS']}
    #XBOX Wake on Lan
    elif type == XBOX_ONE_WOL:
        data = db.select_one("SELECT * FROM XBOX_ONE_WOL WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON'], 'ip': data['IP_ADDRESS'], 'liveid': data['XBOX_LIVE_ID']}
    #Rademacher
    elif type == RADEMACHER_THERMOSTAT:
        data = db.select_one("SELECT * FROM HOMEPILOT_THERMOSTATS WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'],
                       'icon': data['ICON']}
    elif type == RADEMACHER_BLIND_CONTROL:
        data = db.select_one("SELECT * FROM HOMEPILOT_BLIND_CONTROL WHERE ID = :id", param_array)

        device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                       'icon': data['ICON']}
    else:
        return {'result': 'nosuchtype'}

    return {'devicedata': device_data}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Cronjob import IntervalCronjob
from Homevee.DeviceAPI.heating import heating_control
from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database


class HeatingSchemeCronjob(IntervalCronjob):
    def __init__(self):
        super(HeatingSchemeCronjob, self).__init__(task_name="HeatingScheme", interval_seconds=60)

    def task_to_do(self, *args):
        self.run_heating_scheme()

    def run_heating_scheme(self):
        db = Database()

        current_datetime = datetime.datetime.now()

        #Query heating scheme for this day and time
        query = 'SELECT * FROM HEATING_SCHEME, HEATING_SCHEME_DAYS, HEATING_SCHEME_DEVICES WHERE HEATING_SCHEME.ID = HEATING_SCHEME_ID AND HEATING_SCHEME.ID = HEATING_SCHEME_DEVICES.ID AND ACTIVE = "true" AND TIME = :time AND WEEKDAY_ID = :day;'

        current_time = current_datetime.strftime("%H:%M")
        current_day = current_datetime.weekday()

        #Run command
        results = db.select_all(query, {'time': current_time, 'day': current_day})

        for item in results:
            Logger.log(item)

            heating_control(None, item['TYPE'], item['DEVICE_ID'], item['VALUE'], db, check_user=False)


from platform import system as system_name  # Returns the system/OS name
from subprocess import call   as system_call  # Execute a shell command

from Homevee.Cronjob import IntervalCronjob
from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database


class UserPingCronjob(IntervalCronjob):
    def __init__(self):
        super(UserPingCronjob, self).__init__(task_name="UserPingCronjob", interval_seconds=15)

    def task_to_do(self, *args):
        #self.ping_users()
        pass

    def ping_users(self):
        db = Database()

        items = db.select_all("SELECT * FROM USERDATA", {})

        for item in items:
            ip = item['IP']

            Logger.log("Pinging user: "+item['USERNAME'])

            last_val = item['AT_HOME']

            at_home = self.ping(ip)

            #if last_val != at_home:
            #   trigger automation

            if at_home:
                Logger.log(item['USERNAME']+ ": at home")
            else:
                Logger.log(item['USERNAME']+ ": not at home")

            db.update("UPDATE USERDATA SET AT_HOME = :at_home WHERE USERNAME = :user",
                        {'at_home': at_home, 'user': item['USERNAME']})

    def ping(self, host):
        """
        Returns True if host (str) responds to a ping request.
        Remember that a host may not respond to a ping (ICMP) request even if the host name is valid.
        """

        # Ping command count option as function of OS
        param = '-n' if system_name().lower()=='windows' else '-c'

        # Building the command. Ex: "ping -c 1 google.com"
        command = ['ping', param, '1', host]

        # Pinging
        return system_call(command) == 0


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_all_scenes(user, db: Database = None):
    if db is None:
        db = Database()

    results = db.select_all("SELECT * FROM ROOMS", None)

    rooms = []

    for room in results:
        if not user.has_permission(room['LOCATION']):
            continue

        scenes = get_scenes(user, room['LOCATION'], db)

        scenes = scenes['scenes']

        if(len(scenes) is not 0):
            room_item = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'], 'scenes': scenes}
            rooms.append(room_item)

    return {'rooms': rooms}

def get_scenes(user, location, db: Database = None):
    if db is None:
        db = Database()
    scenes = []

    if isinstance(location, Room):
        location = location.id

    if user.has_permission(location):
        items = db.select_all("SELECT * FROM SCENES WHERE ROOM = :location",{'location': location})

        for item in items:
            scenes.append({'id': item['ID'], 'name': item['NAME'], 'action_data': item['ACTION_DATA'],
                          'location': item['ROOM']})

    return {'scenes': scenes}

def add_edit_scene(username, id, name, location, action_data, db: Database = None):
    if db is None:
        db = Database()
    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        if(add_new):
            db.insert("INSERT INTO SCENES (NAME, ROOM, ACTION_DATA) VALUES (:name, :room, :actions)",
                        {'name': name, 'room': location, 'actions': action_data})

            return Status(type=STATUS_OK).get_dict()

        else:
            db.update("UPDATE SCENES SET NAME = :name, ROOM = :location, ACTION_DATA = :actions WHERE ID = :id",
                {'name': name, 'location': location, 'actions': action_data, 'id': id})

            return Status(type=STATUS_OK).get_dict()

def delete_scene(username, id, db: Database = None):
    if db is None:
        db = Database()
    with db:
        cur = db.cursor()

        db.delete("DELETE FROM SCENES WHERE ID = :id", {'id': id})

        return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Cronjob import IntervalCronjob
from Homevee.DeviceAPI import philips_hue
from Homevee.Utils.Database_NEW import Database


class PhilipsHueValueLoaderCronjob(IntervalCronjob):
    def __init__(self):
        super(PhilipsHueValueLoaderCronjob, self).__init__(task_name="PhilipsHueValueLoaderCronjob", interval_seconds=5*60)

    def task_to_do(self, *args):
        self.load_hue_values()

    def load_hue_values(self):
        db = Database()

        results = db.select_all("SELECT * FROM PHILIPS_HUE_LIGHTS", {})
        for item in results:
            counter = 3
            for i in range(0, counter):
                if(philips_hue.get_light_info(item['ID'], db)):
                    break


import traceback

from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database


class HomeBudgetItem(Item):
    def __init__(self, date, info, amount, id=None):
        super(HomeBudgetItem, self).__init__()
        self.date = date
        self.info = info
        self.amount = amount
        self.id = id

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM HOME_BUDGET_DATA WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                new_id = db.insert("""INSERT INTO HOME_BUDGET_DATA (DATE, INFO, AMOUNT) VALUES 
                (:date, :info, :amount)""", {'date': self.date, 'info': self.info, 'amount': self.amount})
                self.id = new_id
            # update
            else:
                db.update("""UPDATE HOME_BUDGET_DATA SET DATE = :date, INFO = :info, AMOUNT = :amount 
                WHERE ID = :id""", {'date': self.date, 'info': self.info, 'amount': self.amount, 'id': self.id})
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save homebudgetitem to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'info': self.info,
            'date': self.date,
            'amount': self.amount
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = HomeBudgetItem(result['DATE'], result['INFO'], result['AMOUNT'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return HomeBudgetItem.load_all_from_db('SELECT * FROM HOME_BUDGET_DATA WHERE ID IN (%s)'
                                               % ','.join('?' * len(ids)), ids)

    @staticmethod
    def load_all(db: Database):
        if db is None:
            db = Database()
        return HomeBudgetItem.load_all_from_db("""SELECT * FROM HOME_BUDGET_ITEM""", {}, db)

    @staticmethod
    def load_home_budget_items_by_date(date, db: Database = None):
        if db is None:
            db = Database()
        return HomeBudgetItem.load_all_from_db("""SELECT * FROM HOME_BUDGET_DATA WHERE DATE = :date
         GROUP BY DATE ORDER BY DATE ASC""", {'date': date}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            date = dict['date']
            info = dict['info']
            amount = dict['amount']

            item = HomeBudgetItem(date, info, amount, id)

            return item
        except:
            raise InvalidParametersException("HomeBudgetItem.create_from_dict(): invalid dict")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback

from Homevee.DeviceAPI import rademacher_homepilot
from Homevee.Helper import Logger
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


def set_blinds(user, type, id, new_position, db, check_user=True):
    try:
        if type == RADEMACHER_BLIND_CONTROL:
            data = db.select_one("SELECT LOCATION FROM HOMEPILOT_BLIND_CONTROL WHERE ID = :id", {'id': id})
            if check_user and not user.has_permission(data['LOCATION']):
                return Status(type=STATUS_NO_PERMISSION).get_dict()

            return rademacher_homepilot.blinds_control.control_blinds(id, new_position)
        else:
            return Status(type=STATUS_NO_SUCH_TYPE).get_dict()
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
        return Status(type=STATUS_ERROR).get_dict()

def set_room_blinds(user, room, new_position, db: Database = None):
    if db is None:
        db = Database()

        devices = get_blinds(user, room)
        
        for device in devices['blinds']:
            id = device['id']
            type = device['type']
            result = set_blinds(user, type, id, new_position, db)
            if 'result' not in result or result['result'] != 'ok':
                return Status(type=STATUS_ERROR).get_dict()
        return Status(type=STATUS_OK).get_dict()
        
def get_all_blinds(user, db: Database = None):
    if db is None:
        db = Database()

        rooms = db.select_all("SELECT * FROM ROOMS")
        blinds = []
        for room in rooms:
            if not user.has_permission(room['LOCATION']):
                continue
            room_blinds = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'],
                                'blind_array': get_blinds(user, room['LOCATION'])}
            blinds.append(room_blinds)
        return {'blinds': blinds}

def get_blinds(user, location, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(location):
        return {'result': 'nopermission'}

    if isinstance(location, Room):
        location = location.id

    blinds = []

    #Rademacher HomePilot
    items = db.select_all("SELECT * FROM HOMEPILOT_BLIND_CONTROL WHERE LOCATION = :location",
                {'location': location})
    for item in items:
        value = int(item['LAST_POS'])

        if value is None:
            value = 0

        blinds.append({'name': item['NAME'], 'id': item['ID'], 'location': location,
                       'icon': item['ICON'], 'value': value, 'type': RADEMACHER_BLIND_CONTROL})

    #Andere Gerätetypen

    return {'blinds': blinds}



#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import io
import json
import os
import time
import traceback
from _thread import start_new_thread

from PIL import Image

from Homevee.DeviceAPI.get_modes import get_modes
from Homevee.DeviceAPI.heating import get_thermostat_info
from Homevee.DeviceAPI.rgb_control import get_rgb_device
from Homevee.DeviceAPI.set_modes import set_modes
from Homevee.Functions.condition_actions.actions import run_scene
from Homevee.Functions.tensorflow_functions import ar_control_predict, ar_control_retrain
from Homevee.Helper import Logger
from Homevee.Item.Status import *
from Homevee.Utils import Constants
from Homevee.Utils.Constants import DATA_DIR
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *
from Homevee.Utils.FileUtils import create_image

IS_TRAINING_TAG = "AR_CONTROL_TRAINING_RUNNING"

def upload_images(user, data, img_class, db: Database = None):
    if db is None:
        db = Database()
    images = json.loads(data)

    counter = 0

    if (img_class == "-1" or img_class == -1):
        img_class = None

    for image in images:
        filename = "device-" + str(int(time.time())) + "_" + str(counter)

        counter += 1

        image_path = create_image(filename, "ar_control", image, optimize=True)

        db.insert("INSERT INTO AR_CONTROL_LEARNING_DATA (PATH, CONTROL_KEY) VALUES (:path, :class)",
                    {'path': image_path, 'class': img_class})
        #print image

    return Status(type=STATUS_OK).get_dict()

def save_ar_control_class(user, id, data, classname, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return {'result': 'noadmin'}

    params = {'name': classname, 'data': data}

    if id is not None and id is not "" and id != "-1":
        params['id'] = id
        db.update("UPDATE AR_CONTROL_CLASSES SET NAME = :name, CONTROL_DATA = :data WHERE ID = :id",
                    params)
    else:
        db.insert("INSERT INTO AR_CONTROL_CLASSES (NAME, CONTROL_DATA) VALUES (:name, :data)",
                    params)

    return Status(type=STATUS_OK).get_dict()

def ar_control(user, image_data, db: Database = None):
    if db is None:
        db = Database()
    filename = "device-"+str(int(time.time()))

    image_path = create_image(filename, "ar_control", image_data, optimize=True)

    db.insert("INSERT INTO AR_CONTROL_LEARNING_DATA (PATH) VALUES (:path)",
               {'path': image_path})

    if not has_ar_control_classes(db):
        return "Noch nicht trainiert."

    predictions = ar_control_predict.predict(os.path.join(Constants.DATA_DIR, image_path))

    class_data = get_ar_class_data(predictions[0]['prediction'])

    Logger.log(image_path)
    Logger.log(class_data['NAME'] + ": " + str(predictions[0]['confidence']*100))
    #Logger.log(class_data)

    data = run_ar_command(user, class_data['CONTROL_DATA'])

    return data

def has_ar_control_classes(db):
        result = db.select_one("SELECT COUNT(*) FROM AR_CONTROL_CLASSES")
        count = result['COUNT(*)']

        return count > 0

def run_ar_command(user, data, db: Database = None):
    if db is None:
        db = Database()
    data = json.loads(data)

    if data['type'] == "scene":
        run_scene(user, data['id'])
    elif data['type'] in [FUNKSTECKDOSE, URL_TOGGLE, URL_SWITCH, ZWAVE_SWITCH]:
        mode = get_modes(user, None, data['type'], data['id'])

        if mode is 0:
            mode = 1
        else:
            mode = 0

        set_modes(user, data['type'], data['id'], str(mode), db)

        return {'action': None}
    elif data['type'] in [MAX_THERMOSTAT, ZWAVE_THERMOSTAT]:
        thermostat_data = get_thermostat_info(user, None, data['type'], data['id'])
        return {'action': 'heating', 'data': thermostat_data, 'type': data['type'], 'id': data['id']}
    elif data['type'] in [PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]:
        rgb_data = get_rgb_device(user, data['type'], data['id'])
        return {'action': 'rgb', 'data': rgb_data, 'type': data['type'], 'id': data['id']}
    else:
        return {'error': 'nosuchaction'}

def get_ar_class_data(id, db: Database = None):
    if db is None:
        db = Database()
    result = db.select_one("SELECT * FROM AR_CONTROL_CLASSES WHERE ID = :id;", {'id': id})
    return result

def start_ar_training(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return {'result': 'noadmin'}

    is_training = db.get_server_data(IS_TRAINING_TAG)

    if (is_training is not None and is_training == "true"):
        return {'result': 'alreadytraining'}

    start_new_thread(training_thread, (user, None))

    return Status(type=STATUS_OK).get_dict()


def training_thread(user, d):
    db = Database()

    db.set_server_data(IS_TRAINING_TAG, "true")

    try:
        ar_control_retrain.ar_training(user)
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

    db.set_server_data(IS_TRAINING_TAG, "false")

def get_ar_control_classes(db):
        results = db.select_all("SELECT * FROM 'AR_CONTROL_CLASSES'")

        classes = []

        for row in results:
            item = {'id': int(row['ID']), 'name': row['NAME'], 'data': row['CONTROL_DATA']}
            classes.append(item)

        item = {'id': None, 'name': 'Nicht zugeordnet'}
        classes.append(item)

        return {'classes': classes}

def get_ar_control_class_images(image_class, show, offset, db: Database = None):
    if db is None:
        db = Database()
    param_array = {'limit': show, 'offset': offset}

    if image_class is None or image_class == "-1" or image_class == -1:
        results = db.select_all("SELECT * FROM 'AR_CONTROL_LEARNING_DATA' WHERE CONTROL_KEY IS NULL OR CONTROL_KEY = -1 LIMIT :limit OFFSET :offset",
                    param_array)
    else:
        param_array['key'] = image_class
        results = db.select_all("SELECT * FROM 'AR_CONTROL_LEARNING_DATA' WHERE CONTROL_KEY = :key LIMIT :limit OFFSET :offset",
                    param_array)

    images = []

    for img in results:
        images.append(img['ID'])

    return {'images': images}

def change_ar_image_class(ids, newclass, db: Database = None):
    if db is None:
        db = Database()
    if newclass == "-1":
        newclass = None

    ids = json.loads(ids)

    Logger.log(ids)

    for id in ids:
        db.update("UPDATE 'AR_CONTROL_LEARNING_DATA' SET CONTROL_KEY = :key WHERE ID == :id;",
                    {'key': newclass, 'id': id})

    return Status(type=STATUS_OK).get_dict()

def delete_ar_control_images(ids, db: Database = None):
    if db is None:
        db = Database()
    #todo

    ids = json.loads(ids)

    for id in ids:
        Logger.log("deleting: "+str(id))
        item = db.select_one("SELECT * FROM AR_CONTROL_LEARNING_DATA WHERE ID == :id;",
                    {'id': id})
        if(item is not None):
            rel_path = item['PATH']

            image_path = os.path.join(DATA_DIR, rel_path)
            if os.path.exists(image_path):
                os.remove(image_path)
            else:
                Logger.log(image_path+" not found")

    for id in ids:
        db.delete("DELETE FROM AR_CONTROL_LEARNING_DATA WHERE ID == :id;",
                    {'id': id})

    return Status(type=STATUS_OK).get_dict()

def get_ar_control_image(id, db: Database = None):
    if db is None:
        db = Database()
    item = db.select_one("SELECT PATH FROM 'AR_CONTROL_LEARNING_DATA' WHERE ID == :id",
                {'id': id})

    rel_path = item['PATH']

    path = os.path.join(DATA_DIR, rel_path)
    Logger.log(("Path: "+path))

    try:
        im = Image.open(path)
    except:
        return {'imagedata': None}

    size = 50

    image_dimensions = size, size

    im.thumbnail(image_dimensions, Image.ANTIALIAS)

    buffer = io.BytesIO()
    im.save(buffer, format="JPEG")
    encoded_string = base64.b64encode(buffer.getvalue())
    encoded_string = encoded_string.decode('utf-8')
    im.close()

    #with open(path, "rb") as image_file:
    #    encoded_string = base64.b64decode(image_file.read())

    return {'imagedata': encoded_string}

def get_performance_settings(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    prefix = "AR_"

    tf_data = {}

    indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

    for index in indices:
        result = db.get_server_data(prefix+index)

        if(result is not None and 'VALUE' in result):
            tf_data[index] = result['VALUE']

    return {'data': tf_data, 'graph_sizes': ar_control_retrain.get_graph_sizes(),
            'image_sizes' : ar_control_retrain.get_image_sizes()}

def set_performance_settings(user, data, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission('admin'):
        return {'result': 'noadmin'}

    prefix = "AR_"

    data = json.loads(data)

    indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

    for index in indices:
        param_array = {'value': data[index], 'key': prefix+index}

        db.update("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key;", param_array)

        db.insert("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES (:value, :key);", param_array)

    return {'result':'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Exception import ItemNotFoundException
from Homevee.Item.Status import *
from Homevee.Item.User import User, Permission
from Homevee.Utils.Database_NEW import Database


def set_user_fcm_token(user, token, db: Database = None):
    if db is None:
        db = Database()
    user.fcm_token = token

    try:
        user.save_to_db(db)
        return Status(type=STATUS_OK).get_dict()
    except:
        return Status(type=STATUS_ERROR).get_dict()

def has_users(db):
    #with db:
    #    # Nutzer laden
    #    cur = db.cursor()
    #    cur.execute("SELECT COUNT(*) FROM USERDATA")
    #    result = cur.fetchone()
    #    if result['COUNT(*)'] > 0:
    #        return True
    #    else:
    #        return False

    users = User.load_all(db)
    return len(users) > 0

def get_users(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.hash_password("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    users = User.load_all(db)
    return {'userdata': User.list_to_dict(users)}

def delete_user(user, user_to_delete, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    user_to_delete = User.load_username_from_db(user_to_delete, db)
    return user_to_delete.api_delete()

def add_edit_user(user, name, psw, ip, permissions, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        if user.username != name:
            return Status(type=STATUS_NO_PERMISSION).get_dict()

    hashed_pw, salt = User.hash_password(psw)

    try:
        edit_user = User.load_username_from_db(name, db)

        if not (psw == "" or psw is None):
            edit_user.hashed_password = hashed_pw
            edit_user.salt = salt

        edit_user.ip = ip
        edit_user.permissions = Permission.create_list_from_json(permissions)

    except ItemNotFoundException:
        edit_user = User(username=name, hashed_password=hashed_pw, salt=salt, ip=ip,
                         permissions=Permission.create_list_from_json(permissions),
                         at_home=False, fcm_token=None, current_location=None,
                         dashboard_data=None, events_last_checked=0)

    return edit_user.api_save()


#!flask/bin/python
from flask import Blueprint, request

from Homevee.API import API
from Homevee.Utils.Database_NEW import Database

ProcessDataAPI = Blueprint('ProcessDataAPI', __name__, template_folder='templates')

@ProcessDataAPI.route('/processdata/<data>', methods=['GET'])
def is_premium(data):
    print("processdata")

    assert data == request.view_args['data']

    msg = API().process_data(data, Database())
    return msg


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item import Item
from Homevee.Item.ShoppingListItem import ShoppingListItem
from Homevee.Utils.Database_NEW import Database


def get_shopping_list(user, db: Database = None):
    if db is None:
        db = Database()
    items = ShoppingListItem.load_all(db)
    return ShoppingListItem.list_to_dict(items)

def add_edit_shopping_list_item(user, id, amount, name, db: Database = None):
    if db is None:
        db = Database()
    shopping_list_item = Item.load_from_db(ShoppingListItem, id, db)
    if shopping_list_item is None:
        shopping_list_item = ShoppingListItem(name, amount, id)
    else:
        shopping_list_item.amount = amount
        shopping_list_item.name = name

    return shopping_list_item.api_save()

def delete_shopping_list_item(user, id, db: Database = None):
    if db is None:
        db = Database()
    item = Item.load_from_db(ShoppingListItem, id, db)
    return item.api_delete()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import re
import urllib.error
import urllib.parse
import urllib.request

from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Helper import generate_string
from Homevee.VoiceAssistant.Modules import VoiceModule

MIN_OUTPUT_LENGTH = 100

class VoiceGetWikipediaDefinitionModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['wer', 'was'], ['ist', 'war']]
        ]

    def get_label(self):
        return "getwikipediadefinition"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.voice_definition(username, text, context, db)

    def voice_definition(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        words = text.split(" ")

        definition = []

        query = None

        is_query = False

        for word in words:
            if is_query:
                if query is None:
                    query = word.capitalize()
                else:
                    query = query + " " + word.capitalize()

            if word == "ist" or word == "war":
                is_query = True

        url = 'https://de.wikipedia.org/w/api.php?format=json&action=query&prop=extracts&exintro=&explaintext=&titles='+query.replace(" ", "%20")
        response = urllib.request.urlopen(url).read()

        Logger.log(url)
        Logger.log(response)

        data = json.loads(response)

        pages = data['query']['pages']

        page_keys = list(pages.keys())[0]

        if 'extract' in pages[page_keys]:
            definition = pages[page_keys]['extract']

        if  page_keys == -1 or len(definition) == 0:
            answers = [
                ['Ich habe ', ['zu ', 'über '], query, ' ' , ['leider ', 'bedauerlicherweise ', ''], [[['gar ', ''], 'nichts '], 'keine Ergebnisse '], ['gefunden', 'finden können'], '.']
            ]

            output = generate_string(answers)
            return {'msg_speech': output, 'msg_text': output}

        definition = definition.replace('[[', '')
        definition = definition.replace(']]', '')
        definition = definition.replace('\n', '')

        definition = re.sub("\s*[\(\[].*?[\)\]]\s*", " ", definition)
        definition = re.sub("\s+", " ", definition)

        #definition = re.sub('#\s*\(.+\)\s*#U', ' ', definition)
        #definition = re.sub('#\s*\[.+\]\s*#U', ' ', definition)

        sentences = definition.split('. ')

        output = sentences[0]

        last_char = output[-1:]

        index = 1
        if len(sentences) > 1:
            while self.is_number(last_char) or len(output) < MIN_OUTPUT_LENGTH:
                output = output + '. ' + sentences[index]
                index = index + 1
                last_char = output[-1:]

        if not output.endswith("."):
            output = output + "."

        return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/env python
# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#		 http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import os
from operator import itemgetter

import numpy as np

try:
    import tensorflow as tf
    TENSORFLOW_INSTALLED = True
    #Logger.log("Tensorflow imported")
except:
    TENSORFLOW_INSTALLED = False
    #Logger.log("Error importing Tensorflow")

from Homevee.Utils import Constants


def load_graph(model_file):
    graph = tf.Graph()
    graph_def = tf.GraphDef()

    with open(model_file, "rb") as f:
        graph_def.ParseFromString(f.read())
    with graph.as_default():
        tf.import_graph_def(graph_def)

    return graph


def read_tensor_from_image_file(file_name, input_height=299, input_width=299,
                                input_mean=0, input_std=255):
    input_name = "file_reader"
    output_name = "normalized"
    file_reader = tf.read_file(file_name, input_name)
    if file_name.endswith(".png"):
        image_reader = tf.image.decode_png(file_reader, channels=3, name='png_reader')
    elif file_name.endswith(".gif"):
        image_reader = tf.squeeze(tf.image.decode_gif(file_reader, name='gif_reader'))
    elif file_name.endswith(".bmp"):
        image_reader = tf.image.decode_bmp(file_reader, name='bmp_reader')
    else:
        image_reader = tf.image.decode_jpeg(file_reader, channels=3, name='jpeg_reader')
    float_caster = tf.cast(image_reader, tf.float32)
    dims_expander = tf.expand_dims(float_caster, 0);
    resized = tf.image.resize_bilinear(dims_expander, [input_height, input_width])
    normalized = tf.divide(tf.subtract(resized, [input_mean]), [input_std])
    sess = tf.Session()
    result = sess.run(normalized)

    return result

def load_labels(label_file):
    label = []
    proto_as_ascii_lines = tf.gfile.GFile(label_file).readlines()
    for l in proto_as_ascii_lines:
        label.append(l.rstrip())
    return label

def predict(file_name):

    file_name = os.path.join(Constants.DATA_DIR, file_name)

    BASE_DIR = os.path.dirname(os.path.abspath(__file__))
    model_dir = os.path.join(BASE_DIR, "tensorflow_data", "models", "mobilenet_v1_1.0_224")
    temp_dir = os.path.join(BASE_DIR, "tensorflow_data", "ar_control_temp")

    model_file = os.path.join(temp_dir, "output_graph.pb")
    label_file = os.path.join(temp_dir, "output_labels.txt")

    input_height = 224
    input_width = 224
    input_mean = 0
    input_std = 255
    input_layer = "input"
    output_layer = "final_result"

    #output_layer = "output"

    '''if args.graph:
        model_file = args.graph
    if args.labels:
        label_file = args.labels
    if args.input_layer:
        input_layer = args.input_layer
    if args.output_layer:
        output_layer = args.output_layer'''

    graph = load_graph(model_file)
    t = read_tensor_from_image_file(file_name, input_height=input_height, input_width=input_width,
                                    input_mean=input_mean, input_std=input_std)

    input_name = "import/" + input_layer
    output_name = "import/" + output_layer
    input_operation = graph.get_operation_by_name(input_name);
    output_operation = graph.get_operation_by_name(output_name);

    with tf.Session(graph=graph) as sess:
        results = sess.run(output_operation.outputs[0], {input_operation.outputs[0]: t})
    results = np.squeeze(results)

    top_k = results.argsort()[-5:][::-1]
    labels = load_labels(label_file)

    max_val = -1
    max_label = -1

    predictions = []

    for i in top_k:
        #Logger.log(labels[i], results[i])

        predictions.append({'prediction': labels[i], 'confidence': results[i]*100})

        if (results[i] > max_val):
            max_val = results[i]
            max_label = labels[i]

    #sorting descending by confidence
    sorted_predictions = sorted(predictions, key=itemgetter('confidence'), reverse=True)

    #max_val = max_val * 100

    #return (max_label, max_val)

    return sorted_predictions


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json
import random

from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetWeatherModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['wie',['ist','wird'],'wetter'],
            ['regnet','es'],
            ['gibt','regen'],
            ['wird','regnen']
        ]

    def get_label(self):
        return "getweather"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_weather(username, text, context, db)

    def get_weather(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        weather = db.get_server_data('WEATHER_CACHE')
        weather = json.loads(weather)
        city_name = weather['city']['name']
        weather_list = weather['list']

        now = datetime.datetime.now()

        cur_weekday = now.weekday()

        search_weekday_word, search_weekday = self.get_weekday(text, cur_weekday)

        if (context is not None and search_weekday is None):
            return False

        if search_weekday is None:
            #No day given => output tdays weather
            search_weekday_word = "Heute"
            search_weekday = cur_weekday

        search_daytime = self.get_day_time(text)

        #current day after search day?
        if search_weekday < cur_weekday:
            #add one week to search day, since its next week
            search_weekday += 7

        day_difference = search_weekday - cur_weekday

        #add the difference
        search_date = now + datetime.timedelta(days=day_difference)

        output = None

        for day in weather_list:
            cur_date = datetime.datetime.fromtimestamp(day['dt'])
            if (cur_date.date() == search_date.date()):
                single_time = False
                single_time_string = None
                if search_daytime is not None:
                    single_time = True
                    temp = int(day['temp'][search_daytime])

                    data = [
                        ['bei ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad'],
                        ['und die Temperatur liegt bei ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad'],
                        ['und die Temperatur liegt ', ['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' bei ', ' ', temp, ' Grad'],
                        ['und es wird ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad haben'],
                        ['und es hat ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad']
                    ]
                    temp_string = Helper.generate_string(data)

                    time_strings = {
                        'morn': ['früh', 'in der Früh'],
                        'day': ['tagsüber'],
                        'eve': ['Abend', 'am Abend'],
                        'night': ['in der Nacht', 'nachts']
                    }
                    single_time_string = random.choice(time_strings[search_daytime])
                else:
                    temp = {'min': int(day['temp']['min']), 'max': int(day['temp']['max'])}

                    if(temp['min'] == temp['max']):
                        data = [
                            ['und die Temperatur ', ['beträgt', 'liegt bei'], ' '+str(temp['max'])+' Grad'],
                            ['bei '+str(temp['min'])+' Grad'],
                            ['bei einer Temperatur von '+str(temp['max'])+' Grad']
                        ]
                    else:
                        data = [
                            ['und die Temperaturen ', ['betragen', 'liegen'], ' zwischen '+str(temp['min'])+' und '+str(temp['max'])+' Grad'],
                            ['bei '+str(temp['min'])+' bis '+str(temp['max'])+' Grad'],
                            ['bei Temperaturen zwischen '+str(temp['min'])+' und '+str(temp['max'])+' Grad']
                        ]
                    temp_string = Helper.generate_string(data)

                weather_ids = []

                for weather in day['weather']:
                    weather_ids.append(weather['id'])

                weather_descs = self.get_weather_description(weather_ids)

                weather_descs_string = None
                for i in range(0, len(weather_descs)):
                    if weather_descs_string is None:
                        weather_descs_string = weather_descs[i]
                    elif i == len(weather_descs)-1:
                        weather_descs_string = ' und '+weather_descs[i]
                    else:
                        weather_descs_string = ', '+weather_descs[i]

                if day_difference <= 2:
                    day_desc_words = ['Heute', 'Morgen', 'Übermorgen']
                    day_string = day_desc_words[day_difference]

                    if single_time_string is not None:
                        day_string += ' '+single_time_string

                else:
                    if single_time_string is None:
                        day_string = Helper.generate_string([
                            ['Am '+search_weekday_word],
                            [search_weekday_word+'s']
                        ])
                    else:
                        day_string = search_weekday_word+' '+single_time_string

                output_data = [
                    [day_string + ' ', ['gibt', 'hat'] ,' es ' + weather_descs_string + ' ' + temp_string + '.']
                ]
                output = Helper.generate_string(output_data)

                Logger.log("set context")
                Helper.set_context(username, self.get_context_key(), {}, db)

        return {'msg_speech': output, 'msg_text': output}

    def get_weekday(self, text, cur_weekday):
        words = text.split(' ')

        #normal weekday
        days = ['montag', 'dienstag', 'mittwoch', 'donnerstag', 'freitag', 'samstag', 'sonntag']
        for i in range(0, len(days)):
            day = days[i]
            if day in words:
                return day.capitalize(), i

        day_words = ['heute', 'morgen', 'übermorgen']
        for i in range(0, len(day_words)):
            day = day_words[i]
            if day in words:
                output_day = cur_weekday + i

                if output_day >= 7:
                    #subtract one week
                    output_day -= 7

                return day.capitalize(), output_day

        return None, None

    def get_day_time(self, text):
        day_times = [
            {'time': 'morn', 'keywords': ['morgens', 'früh']},
            {'time': 'day', 'keywords': ['tagsüber']},
            {'time': 'eve', 'keywords': ['abends', 'abend']},
            {'time': 'night', 'keywords': ['nachts', 'nacht']}
        ]

        words = text.split(' ')

        for day_time in day_times:
            for keyword in day_time['keywords']:
                if keyword in words:
                    return day_time['time']

        return None

    def get_weather_description(self, codes):
        description_map = {
            #Gewitter
            200: 'Gewitter mit leichtem Regen',
            201: 'Gewitter mit Regen',
            202: 'Gewitter mit starkem Regen',
            210: 'leichte Gewitter',
            211: 'Gewitter',
            212: 'starke Gewitter',
            221: 'Gewitter mit leichtem Regen',
            230: 'Gewitter mit leichtem Regen',
            231: 'Gewitter mit leichtem Regen',
            232: 'Gewitter mit leichtem Regen',

            #Nieselregen
            300: 'Nieselregen',
            301: 'Nieselregen',
            302: 'Nieselregen',
            310: 'Nieselregen',
            311: 'Nieselregen',
            312: 'Nieselregen',
            313: 'Nieselregen',
            314: 'Nieselregen',
            321: 'Nieselregen',

            #Regen
            500: 'leichten Regen',
            501: 'mittelstarken Regen',
            502: 'starken Regen',
            503: 'sehr starken Regen',
            504: 'extremen Regen',
            511: 'gefrierenden Regen',
            520: 'leichte Schauer',
            521: 'Schauer',
            522: 'starke Schauer',
            531: 'starke Schauer',

            #Schnee
            600: 'leichten Schneefall',
            601: 'Schneefall',
            602: 'starken Schneefall',
            611: 'Schneeregen',
            612: 'Schneeregen-Schauer',
            615: 'leichten Regen und Schnee',
            616: 'Regen und Schnee',
            620: 'leichte Schneeschauer',
            621: 'Schneeschauer',
            622: 'starke Schneeschauer',

            #Atmosphäre
            701: 'Nebel',
            711: 'Rauch',
            721: 'Dunst',
            731: 'Staubwirbel',
            741: 'Nebel',
            751: 'Sand',
            761: 'Staub',
            762: 'Vulkanasche',
            771: 'Böen',
            781: 'Tornado',

            #Klar
            800: 'klaren Himmel',

            #Wolken
            801: 'wenige Wolken',
            802: 'aufgelockerte Wolken',
            803: 'aufgelockerte Wolken',
            804: 'bedeckte Bewölkung'
        }

        output = []

        for code in codes:
            if code in description_map:
                output.append(description_map[code])

        return output


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Functions.condition_actions.actions import run_actions
from Homevee.Functions.condition_actions.conditions import conditions_true
from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database


class Automation():
    def run_trigger_automation(self, trigger_type, type, id, value, db: Database = None):
        if db is None:
            db = Database()
        return

    def run_automations(self, automations, db: Database = None):
        if db is None:
            db = Database()
        for item in automations:
            Logger.log(item)
            if (conditions_true(json.loads(item['CONDITION_DATA']), db)):
                if (int(item['TRIGGERED']) == 0):
                    Logger.log("running item: "+str(item))
                    run_actions(json.loads(item['ACTION_DATA']), db)
                    self.set_triggered(item['ID'], 1, db)
            else:
                self.set_triggered(item['ID'], 0, db)

    def set_triggered(self, id, triggered, db: Database = None):
        if db is None:
            db = Database()


            db.update("UPDATE AUTOMATION_DATA SET TRIGGERED = :triggered WHERE ID = :id",
                        {'triggered': triggered, 'id': id})


#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback
import urllib.error
import urllib.parse
import urllib.request

from Homevee.DeviceAPI import zwave
from Homevee.Helper import Logger
from Homevee.Item.Device import Device
from Homevee.Item.Device.Switch.Funksteckdose import Funksteckdose
from Homevee.Item.Status import *
from Homevee.Utils.DeviceTypes import *


def set_modes(user, type, id, mode, db, check_user=True):
    if type == FUNKSTECKDOSE:
        return set_socket(user, id, mode, db, check_user)
    elif type == ZWAVE_SWITCH:
        return set_zwave_switch(user, id, mode, db, check_user)
    elif type == URL_SWITCH:
        return set_url_switch_binary(user, id, mode, db, check_user)
    elif type == URL_TOGGLE:
        return set_url_toggle(user, id, db, check_user)
    else:
        return Status(type=STATUS_NO_SUCH_TYPE).get_dict()

def set_url_switch_binary(user, id, mode, db, check_user=True):
    data = db.select_one("SELECT * FROM URL_SWITCH_BINARY WHERE ID == :id",
                {'id': id})

    if check_user and not user.has_permission(data['LOCATION']):
        return {'result': 'nopermission'}

    if mode == "":
        urllib.request.urlopen(data['URL_ON'])
    else:
        urllib.request.urlopen(data['URL_OFF'])

    return Status(type=STATUS_OK).get_dict()

def set_url_toggle(user, id, db, check_user=True):
    data = db.select_one("SELECT * FROM URL_TOGGLE WHERE ID == :id",
                {'id': id})

    if check_user and not user.has_permission(data['LOCATION']):
        return {'result': 'nopermission'}

    urllib.request.urlopen(data['TOGGLE_URL'])

    return Status(type=STATUS_OK).get_dict()

def set_socket(user, id, mode, db, check_user=True):
    try:
        funksteckdose = Device.load_from_db(Funksteckdose, id)

        if check_user and not user.has_permission(funksteckdose.location):
            return {'result': 'nopermission'}

        if(funksteckdose.set_mode(mode)):
            return Status(type=STATUS_OK).get_dict()
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
    return Status(type=STATUS_ERROR).get_dict()

def set_zwave_switch(user, id, mode, db, check_user=True):
    try:

        cur = db.cursor()
        data = db.select_one("SELECT * FROM ZWAVE_SWITCHES WHERE ID == :device",
                    {'device': id})

        if check_user and not user.has_permission(data['LOCATION']):
            return {'result': 'nopermission'}

        device_id = data['ID']

        state = "on" if (int(mode) == 1) else "off"

        result = zwave.device_control.set_binary_device(device_id, state)

        if result['code'] == 200:
            db.update("UPDATE ZWAVE_SWITCHES SET VALUE = :value WHERE ID = :id",
                        {'value': mode, 'id': id})

            return Status(type=STATUS_OK).get_dict()
    except Exception as e:
        if Logger.IS_DEBUG:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
        return Status(type=STATUS_ERROR).get_dict()

def set_diy_switch(id, mode, db, check_user=True):
    return


from Homevee.Exception import *
from Homevee.Item.Device.Sensor import Sensor
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import ZWAVE_SENSOR


class ZWaveSensor(Sensor):
    def __init__(self, name, icon, location, save_data, sensor_type, id=None, value=None):
        super(ZWaveSensor, self).__init__(name, icon, location, save_data, sensor_type, id=id, value=value)

    def get_device_type(self):
        return ZWAVE_SENSOR

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM ZWAVE_SENSOREN WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            #insert
            if(self.id is None or self.id == ""):
                db.insert("INSERT INTO ZWAVE_SENSOREN (RAUM, SHORTFORM, ICON, SAVE_DATA,"
                            "SENSOR_TYPE, VALUE) VALUES (:location, :name, :icon, :save_data,"
                            ":sensor_type)",
                            {'location': self.location, 'name': self.name, 'icon': self.icon,
                             'save_data': self.save_data, 'sensor_type': self.sensor_type})
            #update
            else:
                db.update("UPDATE ZWAVE_SENSOREN SET RAUM = :location, SHORTFORM = :name, ICON = :icon,"
                            "SAVE_DATA = :save_data, SENSOR_TYPE = :sensor_type, VALUE = :value"
                            "WHERE ID = :id",
                            {'location': self.location, 'name': self.name, 'icon': self.icon,
                             'save_data': self.save_data, 'sensor_type': self.sensor_type,
                             'value': self.value, 'id': self.id})
                    # TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save room to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'value': self.value,
            'id': self.id,
            'location': self.location,
            'save_data': self.save_data,
            'sensor_type': self.sensor_type
        }
        return dict

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return ZWaveSensor.load_all_from_db('SELECT * FROM ZWAVE_SENSOREN WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = ZWaveSensor(result['SHORTFORM'], result['ICON'], result['RAUM'], result['SAVE_DATA'],
                                 result['SENSOR_TYPE'], result['ID'], result['VALUE'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database):
        if db is None:
            db = Database()
        return ZWaveSensor.load_all_from_db('SELECT * FROM ZWAVE_SENSOREN', {})

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            value = dict['value']
            icon = dict['icon']
            save_data = dict['save_data']
            sensor_type = dict['sensor_type']

            item = ZWaveSensor(name, icon, location, save_data, sensor_type, id, value)

            return item

        except:
            raise InvalidParametersException("Invalid parameters for ZWaveSensor.create_from_dict()")


#!/usr/bin/python
# -*- coding: utf-8 -*-

from pymax.cube import Cube

from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Item.Gateway import Gateway, MAX_CUBE
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_devices(ip):
    thermostats = []
    try:
        with Cube(ip) as cube:
            for room in cube.rooms:
                for device in room.devices:
                    Logger.log(device)
                    thermostat = {'title': device.name, 'id': device.serial, 'room': room.name}

                    thermostats.append(thermostat)
    except:
        thermostats = []

    return thermostats


def set_temp(id, value, db: Database = None):
    if db is None:
        db = Database()
    gateway = Item.load_from_db(Gateway, MAX_CUBE)

    try:
        with Cube(gateway.ip) as cube:
            for room in cube.rooms:
                for device in room.devices:
                    if id == device.serial:
                        result = cube.set_mode_manual(room.room_id, room.rf_address, float(value))

                        Logger.log(result)

                        db.update("UPDATE MAX_THERMOSTATS SET LAST_TEMP = :temp WHERE ID == :id",
                                    {'temp': value, 'id': id})

                        return Status(type=STATUS_OK).get_dict()
    except:
        return Status(type=STATUS_ERROR).get_dict()


from Homevee.Exception import AbstractFunctionCallException
from Homevee.Item.Device import Device
from Homevee.Utils.Database_NEW import Database


class Thermostat(Device):
    def __init__(self, name, icon, location, id=None, temp=None):
        super(Thermostat, self).__init__(name, icon, location, id=id)
        self.temp = temp

    def set_temp(self, temp, db: Database = None):
        if db is None:
            db = Database()
        if(self.update_temp(temp)):
            self.mode = temp
            self.save_to_db(db)
            return True
        return False

    def update_temp(self, temp: float, db=None) -> bool:
        """
        Updates the temperatur of the thermostat
        :param temp: the new temp
        :param db: the database connection
        :return: true if successful, false otherwise
        """
        raise AbstractFunctionCallException("Thermostat.update_mode() is abstract")

    def get_min_max(self) -> tuple:
        """
        Returns a tuple with the temperature range
        :return: the minimum and maximum temperature value
        """
        raise AbstractFunctionCallException("Thermostat.get_min_max() is abstract")

    #@staticmethod
    #def get_all(location=None, db: Database = None):
    #   if db is None:
    #       db = Database()
    #    # get all devices of all types
    #    devices = []
    #    devices.extend(Switch.get_all(location))
    #    return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item import Item
from Homevee.Item.Device import Device
from Homevee.Item.Device.Sensor.MQTTSensor import *
from Homevee.Item.Device.Sensor.ZWaveSensor import *
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Utils.DeviceTypes import *

SENSOR_TYPE_MAP = {
    'temp': {'name': 'Temperatur', 'einheit': '°C', 'einheit_word': 'Grad'},
    'hygro': {'name': 'Luftfeuchtigkeit', 'einheit': '%', 'einheit_word': '%'},
    'helligkeit': {'name': 'Helligkeit', 'einheit': 'Lux', 'einheit_word': 'Lumen'},
    'uv': {'name': 'UV-Licht', 'einheit': 'UV-Index', 'einheit_word': 'UV-Index'},
    'powermeter': {'name': 'Stromverbrauch', 'einheit': 'Watt', 'einheit_word': 'Watt'},
}


def get_einheit(type, id, db: Database = None):
    if db is None:
        db = Database()
    db_data = {
        ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'sensor_type_col': 'SENSOR_TYPE', 'id_col': 'ID'},
        MQTT_SENSOR: {'table': 'MQTT_SENSORS', 'sensor_type_col': 'TYPE', 'id_col': 'ID'}
    }

    result = db.select_one("SELECT " + db_data[type]['sensor_type_col'] + " FROM " + db_data[type]['table'] + " WHERE " +
                db_data[type]['id_col'] + " = :id",
                {'id': id})

    return SENSOR_TYPE_MAP[result[db_data[type]['sensor_type_col']]]['einheit']

def get_sensor_value(type, id, show_einheit, db: Database = None):
    if db is None:
        db = Database()

    module_map = {
        ZWAVE_SENSOR: ZWaveSensor,
        MQTT_SENSOR: MQTT_SENSOR
    }

    if type not in module_map:
        return "N/A"

    sensor = Device.load_from_db(module_map[type], id)
    output = str(sensor.value)
    if show_einheit:
        output += " " + sensor.get_einheit()

    return output


def get_sensor_data(user, room, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    values = []

    if isinstance(room, Room):
        rooms = [room]
    elif room is None or room == "all":
        rooms = Room.load_all(db)
    else:
        rooms = [Item.load_from_db(Room, room)]

    for room in rooms:
        value_array = []

        if not user.has_permission(room.id):
            continue

        #TODO weitere Sensortypen hinzufügen
        modules = [ZWaveSensor, MQTTSensor]

        for module in modules:
            devices = Device.get_all(module, room)
            for device in devices:
                item = device.get_dict()
                value_array.append(item)

        value_item = {'name': room.name, 'location': room.id, 'icon': room.icon,
                      'value_array': value_array}

        values.append(value_item)

    return {'values': values}



#!/usr/bin/python
# -*- coding: utf-8 -*-

import random

from Homevee.Helper import Logger

my_private = (5385459049, 5090981419)
my_public = (617425929, 5090981419)

'''
Euclid's algorithm for determining the greatest common divisor
Use iteration to make it faster for larger integers
'''
def gcd(a, b):
    while b != 0:
        a, b = b, a % b
    return a


'''
Euclid's extended algorithm for finding the multiplicative inverse of two numbers
'''
def multiplicative_inverse(e, phi):

    d = 0
    x1 = 0
    x2 = 1
    y1 = 1
    temp_phi = phi

    while e > 0:
        temp1 = temp_phi / e
        temp2 = temp_phi - temp1 * e
        temp_phi = e
        e = temp2

        x = x2 - temp1 * x1
        y = d - temp1 * y1

        x2 = x1
        x1 = x
        d = y1
        y1 = y

    if temp_phi == 1:
        return d + phi


'''
Tests to see if a number is prime.
'''
def is_prime(num):
    if num == 2:
        return True
    if num < 2 or num % 2 == 0:
        return False
    for n in range(3, int(num ** 0.5) + 2, 2):
        if num % n == 0:
            return False
    return True


def generate_keypair(p, q):
    if not (is_prime(p) and is_prime(q)):
        raise ValueError('Both numbers must be prime.')
    elif p == q:
        raise ValueError('p and q cannot be equal')
    # n = pq
    n = p * q

    # Phi is the totient of n
    phi = (p - 1) * (q - 1)

    # Choose an integer e such that e and phi(n) are coprime
    e = random.randrange(1, phi)

    # Use Euclid's Algorithm to verify that e and phi(n) are comprime
    g = gcd(e, phi)
    while g != 1:
        e = random.randrange(1, phi)
        g = gcd(e, phi)

    # Use Extended Euclid's Algorithm to generate the private key
    d = multiplicative_inverse(e, phi)

    # Return public and private keypair
    # Public key is (e, n) and private key is (d, n)
    return ((e, n), (d, n))


def encrypt(pk, plaintext):
    # Unpack the key into it's components
    key, n = pk
    # Convert each letter in the plaintext to numbers based on the character using a^b mod m
    cipher = [pow(ord(char),key,n) for char in plaintext]

    for char in plaintext:
        Logger.log((ord(char) ** key) % n)

    # Return the array of bytes
    return cipher


def decrypt(pk, ciphertext):
    # Unpack the key into its components
    key, n = pk
    # Generate the plaintext based on the ciphertext and key using a^b mod m
    plain = [chr(pow(char, key, n)) for char in ciphertext]
    # Return the array of bytes as a string
    return ''.join(plain)


if __name__ == '__main__':
    '''
    Detect if the script is being run directly by the user
    '''
    #print "RSA Encrypter/ Decrypter"
    #p = int(raw_input("Enter a prime number (17, 19, 23, etc): "))
    #q = int(raw_input("Enter another prime number (Not one you entered above): "))
    #print "Generating your public/private keypairs now . . ."
    #public, private = generate_keypair(p, q)
    #print "Your public key is ", public, " and your private key is ", private
    #message = raw_input("Enter a message to encrypt with your private key: ")
    #encrypted_msg = encrypt(private, message)
    #print encrypted_msg
    #print "Your encrypted message is: "
    #print ''.join(map(lambda x: str(x), encrypted_msg))
    #print "Decrypting message with public key ", public, " . . ."
    #print "Your message is:"
    #print decrypt(public, encrypted_msg)

    #cipher = encrypt(my_private, "Hallo ich bin eine interessante und sehr geheime Nachricht")
    #print cipher
    #print decrypt(my_public, cipher)